Package: Ckmeans.1d.dp
Type: Package
Title: Optimal, Fast, and Reproducible Univariate Clustering
Version: 4.3.5
Date: 2023-08-19
Authors@R: 
  c(person("Joe", "Song", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-6883-6547"),
		       email = "joemsong@cs.nmsu.edu"),
	  person("Hua", "Zhong", role = "aut", 
	         comment = c(ORCID = "0000-0003-1962-2603")),
	  person("Haizhou", "Wang", role = "aut"))
Author: Joe Song [aut, cre] (<https://orcid.org/0000-0002-6883-6547>),
  Hua Zhong [aut] (<https://orcid.org/0000-0003-1962-2603>),
  Haizhou Wang [aut]
Maintainer: Joe Song <joemsong@cs.nmsu.edu>
Description: Fast, optimal, and reproducible weighted univariate
 clustering by dynamic programming. Four problems are solved, including
 univariate k-means (Wang & Song 2011) <doi:10.32614/RJ-2011-015>
 (Song & Zhong 2020) <doi:10.1093/bioinformatics/btaa613>, k-median,
 k-segments, and multi-channel weighted k-means. Dynamic programming
 is used to minimize the sum of (weighted) within-cluster distances
 using respective metrics. Its advantage over heuristic clustering in
 efficiency and accuracy is pronounced when there are many clusters.
 Multi-channel weighted k-means groups multiple univariate
 signals into k clusters. An auxiliary function generates histograms
 adaptive to patterns in data. This package provides a powerful set
 of tools for univariate data analysis with guaranteed optimality,
 efficiency, and reproducibility, useful for peak calling on temporal,
 spatial, and spectral data.
License: LGPL (>= 3)
Encoding: UTF-8
Imports: Rcpp, Rdpack (>= 0.6-1)
LinkingTo: Rcpp
NeedsCompilation: yes
Suggests: testthat, knitr, rmarkdown, RColorBrewer
RdMacros: Rdpack
VignetteBuilder: knitr
Packaged: 2023-08-19 17:13:22 UTC; joesong
Repository: CRAN
Date/Publication: 2023-08-19 18:12:31 UTC

useDynLib(Ckmeans.1d.dp, .registration = TRUE)

importFrom(Rcpp, evalCpp)
importFrom(Rdpack, reprompt)

#exportPattern("^[[:alpha:]]+")
export(Ckmeans.1d.dp)
export(Ckmedian.1d.dp)
export(Cksegs.1d.dp)
export(ahist)
export(plotBIC)

export(print.Ckmeans.1d.dp)
S3method(print, Ckmeans.1d.dp)

export(plot.Ckmeans.1d.dp)
S3method(plot, Ckmeans.1d.dp)

export(print.Ckmedian.1d.dp)
S3method(print, Ckmedian.1d.dp)

export(plot.Ckmedian.1d.dp)
S3method(plot, Ckmedian.1d.dp)

export(print.Cksegs.1d.dp)
S3method(print, Cksegs.1d.dp)

export(plot.Cksegs.1d.dp)
S3method(plot, Cksegs.1d.dp)


export(MultiChannel.WUC)

export(plot.MultiChannelClusters)
S3method(plot, MultiChannelClusters)

---
title: The 'Ckmeans.1d.dp' R package
# bibliography: inst/REFERENCES.bib
---

[![Project Status: Active – The project has reached a stable, usable state and is being actively developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/Ckmeans.1d.dp)](https://cran.r-project.org/package=Ckmeans.1d.dp)
[![CRAN_latest_release_date](https://www.r-pkg.org/badges/last-release/Ckmeans.1d.dp)](https://cran.r-project.org/package=Ckmeans.1d.dp)
[![metacran downloads](https://cranlogs.r-pkg.org/badges/Ckmeans.1d.dp)](https://cran.r-project.org/package=Ckmeans.1d.dp)
[![metacran downloads](https://cranlogs.r-pkg.org/badges/grand-total/Ckmeans.1d.dp)](https://cran.r-project.org/package=Ckmeans.1d.dp)

### Overview

The package provides a powerful set of tools for fast, optimal, and reproducible univariate clustering by dynamic programming. It is practical to cluster millions of sample points into a few clusters in seconds using a single core on a typical desktop computer. It solves four types of problem, including univariate $k$-means, $k$-median, $k$-segments, and multi-channel weighted $k$-means. Dynamic programming is used to minimize the (weighted) sum of within-cluster distances using respective metrics. Its advantage over heuristic clustering in efficiency and accuracy is increasingly pronounced as the number of clusters $k$ increases. Weighted $k$-means can also optimally segment time series to perform peak calling. An auxiliary function generates histograms that are adaptive to patterns in data. The package was recently used to map dysregulated zones in genomes of 17 human cancer types (Song and Zhong, 2020).

### The main method

The Ckmeans.1d.dp algorithms cluster (weighted) univariate data given by a numeric vector $x$ into $k$ groups by dynamic programming (Wang and Song, 2011; Zhong, 2019; Song and Zhong, 2020). They guarantee the optimality of clustering---the total of within-cluster sums of squared distances is always minimized given the number of clusters $k$. In contrast, heuristic univariate clustering algorithms may be sub-optimal at large $k$ or inconsistent from run to run. As non-negative weights are supported for each point, the algorithm can also partition a time course using the time points as input and the values at each time point as weight. Utilizing the optimal clusters, a function can generate histograms adaptive to patterns in data.

Excluding the time for sorting $x$, the default weighted univariate clustering algorithm takes a runtime of $O(kn)$ (Song and Zhong, 2020), linear in both sample size $n$ and the number of clusters $k$, using a divide-and-conquer strategy based on a theoretical result on matrix search [(Aggarwal et al., 1987) <doi:10.1007/BF01840359>](https://doi.org/10.1007/BF01840359) implemented via a novel in-place search space reduction method (Song and Zhong, 2020). The space complexity is $O(kn)$. This method is numerically stable.

### When to use the package

As an alternative to popular heuristic clustering methods, this package provides functionality for (weighted) univariate clustering, segmentation, and peak calling with guaranteed optimality and efficiency.

An adaptive histogram based on optimal clusters is also recommended if an equal-bin-width histogram is inadequate to characterize clusters that vary in width.

### To download and install the package
```
install.packages("Ckmeans.1d.dp")
```

### Citing the package

Song M, Zhong H (2020). "Efficient weighted univariate clustering maps outstanding dysregulated genomic zones in human cancers." _Bioinformatics_, 36(20), 5027–5036. 
https://doi.org/10.1093/bioinformatics/btaa613

Wang H, Song M (2011). "Ckmeans.1d.dp: Optimal k-means clustering in one dimension by dynamic programming." _The R Journal_, 3(2), 29–33. https://doi.org/10.32614/RJ-2011-015

## ---- fig.width=6-------------------------------------------------------------
require("Ckmeans.1d.dp")
x <- c(rnorm(40, mean=-2, sd=0.3),
       rnorm(45, mean=1, sd=0.1),
       rnorm(70, mean=3, sd=0.2))
ahist(x, col="lightblue", sub=paste("n =", length(x)),
      col.stick="darkblue", lwd=2, xlim=c(-4,4),
      main="Example 1. Gaussian mixture model with 3 components\n(one bin per component)\nAdaptive histogram")

## ---- fig.width=6-------------------------------------------------------------
ahist(x, breaks=3, col="lightgreen", sub=paste("n =", length(x)),
      col.stick="forestgreen", lwd=2,
      main="Example 1. Regular histogram")

## ---- fig.width=6-------------------------------------------------------------
ahist(x, k=9, col="lavender", col.stick="navy",
      sub=paste("n =", length(x)), lwd=2,
      main="Example 2. Gaussian mixture model with 3 components\n(on average 3 bins per component)\nAdaptive histogram")

## ---- fig.width=6-------------------------------------------------------------
ahist(x, breaks=9, col="lightgreen", col.stick="forestgreen",
      sub=paste("n =", length(x)), lwd=2,
      main="Example 2. Regular histogram")

## ---- fig.show='hold', fig.width=6--------------------------------------------
data(DNase)
res <- Ckmeans.1d.dp(DNase$density)
kopt <- length(res$size)
ahist(res, data=DNase$density, col=rainbow(kopt), col.stick=rainbow(kopt)[res$cluster],
      sub=paste("n =", length(x)), border="transparent",
      xlab="Optical density of protein DNase",
      main="Example 3. Elisa assay of DNase in rat serum\nAdaptive histogram")

## ---- fig.show='hold', fig.width=6--------------------------------------------
ahist(DNase$density, breaks="Sturges", col="palegreen",
      add.sticks=TRUE, col.stick="darkgreen",
      main="Example 3. Elisa assay of DNase in rat serum\nRegular histogram (equal bin width)",
      xlab="Optical density of protein DNase")

## ---- fig.show='hold', fig.width=6--------------------------------------------
x <- c(1,1,1,1, 3,4,4, 6,6,6)
ahist(x, k=c(2,4), col="gray",
      lwd=2, lwd.stick=6, col.stick="chocolate",
      main="Example 4. Adaptive histogram of repetitive elements")
ahist(x, breaks=3, col="lightgreen",
      lwd=2, lwd.stick=6, col.stick="forestgreen",
      main="Example 4. Regular histogram")

## ---- message=FALSE-----------------------------------------------------------
require(Ckmeans.1d.dp)
require(RColorBrewer)

## ---- fig.height=4, fig.width=7-----------------------------------------------
x <- c(rnorm(50, sd=0.3), rnorm(50, mean=1, sd=0.3), rnorm(50, mean=2, sd=0.3))
k <- 3 # Divide x into 3 clusters
result <- Ckmeans.1d.dp(x, k)
colors <- brewer.pal(k, "Dark2")
plot(result, col.clusters = colors)
plot(x, col=colors[result$cluster], pch=result$cluster, cex=1.5,
     main="Optimal univariate clustering given k",
     sub=paste("Number of clusters given:", k))
abline(h=result$centers, col=colors, lty="dashed", lwd=2)
legend("bottomright", paste("Cluster", 1:k), col=colors, pch=1:k, cex=1, bty="n")

## ---- fig.height=4, fig.width=7-----------------------------------------------
x <- c(rnorm(50, mean=-1, sd=0.3), rnorm(50, mean=1, sd=1), rnorm(50, mean=2, sd=0.4))
# Divide x into k clusters, k automatically selected (default: 1~9)
result <- Ckmeans.1d.dp(x)
k <- max(result$cluster)
colors <- brewer.pal(k, "Dark2")
plot(result, col.clusters = colors)
plot(x, col=colors[result$cluster], pch=result$cluster, cex=1.5,
     main="Optimal univariate clustering with k estimated",
     sub=paste("Number of clusters is estimated to be", k))
abline(h=result$centers, col=colors, lty="dashed", lwd=2)
legend("topleft", paste("Cluster", 1:k), col=colors, pch=1:k, cex=1, bty="n")

## ---- fig.height=4, fig.width=7-----------------------------------------------
n <- 160
t <- seq(0, 2*pi*2, length=n)
n1 <- 1:(n/2)
n2 <- (max(n1)+1):n
y1 <- abs(sin(1.5*t[n1]) + 0.1*rnorm(length(n1)))
y2 <- abs(sin(0.5*t[n2]) + 0.1*rnorm(length(n2)))
y <- c(y1, y2)

w <- y^8 # stress the peaks 
res <- Ckmeans.1d.dp(t, k=c(1:10), w)
k <- max(res$cluster)
colors <- brewer.pal(k, "Set1")
plot(res, col.clusters = colors)
grid()
plot(t, w, main = "Time course clustering / peak calling", 
     col=colors[res$cluster], pch=res$cluster, type="h", 
     xlab="Time t", ylab="Transformed intensity w")
grid()
abline(v=res$centers, col="gray", lty="dashed")
text(res$centers, max(w) * .95, cex=0.75, font=2,
     paste(round(res$size / sum(res$size) * 100), "/ 100"))

## ---- fig.height=4, fig.width=7-----------------------------------------------
x <- c(7, 4, 1, 8, 15, 22, -1)
k <- 3
ckm <- Ckmeans.1d.dp(x, k=k)
midpoints <- ahist(ckm, style="midpoints", data=x, plot=FALSE)$breaks[2:k]

colors <- brewer.pal(k, "Set2")
plot(ckm, col.clusters = colors, lwd=5,
     main="Midpoints as cluster boundaries")
abline(v=midpoints, col="RoyalBlue", lwd=3, lty=2)
legend("topright", "Midpoints", lwd=3, lty=2, col="RoyalBlue")

## Ckmeans.1d.dp.R
##
##   Ckmeans.1d.dp()-----Interface funcition to call C++ version kmeans.1d.dp()
##
##   Haizhou Wang
##   Computer Science Department
##   New Mexico State University
##   hwang@cs.nmsu.edu
##
##   Created: Oct 20, 2009
##
## Modified:
#    May 17, 2016. MS
#    September 25, 2016. MS. Introduced function ahist()
#    September 21, 2018. Joe Song. Changed from C to Rcpp interface


##Ckmeans.1d.dp : function which implement optimal one-dimensional clustering
## x is one-dimensional input vector
## k indicates cluster level
cluster.1d.dp <- function( x, k, y, method, estimate.k, criterion, xname, yname )
{
  if(is.null(k)) {
    k <- 1: min( 9, length(x) )
  } else {
    k <- as.integer(ceiling(k))
  }

  if(length(k) > 1) {
    k.min <- min(k)
    k.max <- max(k)
  } else {
    k.min <- k
    k.max <- k
  }

  ##Check to see if k is less than 0.
  ##If k is less than 0, stop and exit.
  if( k.max <= 0 ) {
    stop ("Can NOT classify vector into 0 or less cluster\n")
  }

  n.unique <- length(unique(x))
  ##Check to see if cluster level bigger than the unique number of the input vector
  ##If k is bigger than the unique number of the input vector,
  ##force k set to the number of unique number of input.

  if(n.unique == 0) {
    warning(paste("Input vector", deparse(substitute(x)), "is empty!\n"))
  } else {
    if(n.unique < k.min) {
      warning("Min number of clusters is greater than the unique number of elements in\n",
              "the input vector, both k.min and k.max are set to the number of\n",
              "unique number of input values.\n")
      k.min <- n.unique
      k.max <- n.unique
    } else if (n.unique >= k.min && n.unique < k.max) {
      warning("Max number of clusters is greater than the unique number of\n",
              "elements in the input vector, and k.max is set to the number of\n",
              "unique number of input values.\n")
      k.max <- n.unique
    }
  }

  ##Form data which will be passed to external C++ function.
  clusters <- vector("integer", length(x))
  center <- vector("double", k.max)
  withinss <- vector("double", k.max)
  size <- vector("double", k.max) # vector("integer", k.max)
  BIC <- vector("double", k.max-k.min+1)

  #Call external C++ function
  if(criterion == "L2") {

    # result <- .C("Ckmeans_1d_dp", PACKAGE="Ckmeans.1d.dp",
    #              data=as.double(x), length=as.integer(length(x)),
    #              weight=as.double(y), weight_length=as.integer(length(y)),
    #              Kmin=as.integer(k.min), Kmax=as.integer(k.max),
    #              cluster=as.integer(clusters), centers=as.double(center),
    #              withinss=as.double(withinss), size=as.double(size), # size=as.integer(size),
    #              BIC=as.double(BIC),
    #              estimate.k=as.character(estimate.k),
    #              method=as.character(method))

    result <- Ckmeans_1d_dp(
                 as.double(x), as.integer(length(x)),
                 as.double(y), as.integer(length(y)),
                 as.integer(k.min), as.integer(k.max),
                 as.integer(clusters), as.double(center),
                 as.double(withinss), as.double(size), # size=as.integer(size),
                 as.double(BIC),
                 estimate.k,
                 method)

    class <- "Ckmeans.1d.dp"
  } else if (criterion == "L1") {

    # result <- .C("Ckmedian_1d_dp", PACKAGE="Ckmeans.1d.dp",
    #              data=as.double(x), length=as.integer(length(x)),
    #              weight=as.double(y), weight_length=as.integer(length(y)),
    #              Kmin=as.integer(k.min), Kmax=as.integer(k.max),
    #              cluster=as.integer(clusters), centers=as.double(center),
    #              withinss=as.double(withinss), size=as.double(size), # size=as.integer(size),
    #              BIC=as.double(BIC),
    #              estimate.k=as.character(estimate.k),
    #              method=as.character(method))

    result <- Ckmedian_1d_dp(
      as.double(x), as.integer(length(x)),
      as.double(y), as.integer(length(y)),
      as.integer(k.min), as.integer(k.max),
      as.integer(clusters), as.double(center),
      as.double(withinss), as.double(size), # size=as.integer(size),
      as.double(BIC),
      estimate.k,
      method)

    class <- "Ckmedian.1d.dp"
  } else if(criterion == "L2Y") {

    # result <- .C("Cksegs_1d_dp", PACKAGE="Ckmeans.1d.dp",
    #              data=as.double(x), length=as.integer(length(x)),
    #              weight=as.double(y), weight_length=as.integer(length(y)),
    #              Kmin=as.integer(k.min), Kmax=as.integer(k.max),
    #              cluster=as.integer(clusters), centers=as.double(center),
    #              withinss=as.double(withinss), size=as.double(size), # size=as.integer(size),
    #              BIC=as.double(BIC),
    #              estimate.k=as.character(estimate.k),
    #              method=as.character(method))

    result <- Cksegs_1d_dp(
      as.double(x), as.integer(length(x)),
      as.double(y), as.integer(length(y)),
      as.integer(k.min), as.integer(k.max),
      as.integer(clusters), as.double(center),
      as.double(withinss), as.double(size), # size=as.integer(size),
      as.double(BIC),
      estimate.k,
      method)

    class <- "Cksegs.1d.dp"
  }

  if(length(result$cluster) > 0) {
    k.opt <- max(result$cluster) # length(unique(result$cluster))
  } else {
    k.opt <- 0
  }

  if(n.unique > 0) {
    if(k.min < k.max) {
      if (k.opt == k.min && k.min != 1) {
        warning("Min number of clusters used. Consider decreasing k!\n")
      } else if(k.opt == k.max && k.max != length(x)) {
        warning("Max number of clusters used. Consider increasing k!\n")
      }
    }
  }

  if(length(y) == length(x) && sum(y) != 0 && criterion != "L2Y") {
    totss <- sum(y * (x - sum(as.numeric(x * y)) / sum(y))^2)
  } else if(criterion == "L2Y") {
    totss <- sum((y - sum(as.numeric(y)) / length(y))^2)
  } else {
    # totss <- sum(scale(x, scale=FALSE)^2) scale function is VERY SLOW!
    totss <- sum((x - sum(as.numeric(x)) / length(x))^2)
  }

  tot.withinss <- sum(result$withinss[1:k.opt])
  betweenss <- totss - tot.withinss
  BIC <- result$BIC
  names(BIC) <- paste0("k=", k.min : k.max)

  r <- structure(
    list(
      cluster = result$cluster, centers = result$centers[1:k.opt],
      withinss = result$withinss[1:k.opt], size = result$size[1:k.opt],
      totss = totss, tot.withinss = tot.withinss, betweenss = betweenss,
      BIC = BIC, xname=xname, yname=yname
    ),
    class = class)

  return( r )
} ##end of cluster.1d.dp()


Ckmeans.1d.dp <- function( x, k=c(1,9), y=1,
                           method=c("linear", "loglinear", "quadratic"),
                           estimate.k=c("BIC", "BIC 3.4.12") )
{
  method <- match.arg(method)
  estimate.k <- match.arg(estimate.k)
  cluster.1d.dp(x, k, y, method, estimate.k, "L2",
                deparse(substitute(x)), deparse(substitute(y)))
}

Ckmedian.1d.dp <- function( x, k=c(1,9), y=1,
                            method=c("linear", "loglinear", "quadratic"),
                            estimate.k=c("BIC", "BIC 3.4.12") )
{
  method <- match.arg(method)
  estimate.k <- match.arg(estimate.k)

  if(length(y) > 1) {
    warning("Ckmedian.1d.dp() ignores weight vector y and uses equal weight instead!")
  }

  cluster.1d.dp(x, k, y=1, method, estimate.k, "L1",
                deparse(substitute(x)), deparse(substitute(y)))
}

Cksegs.1d.dp <- function(y, k=c(1,9), x=seq_along(y),
                         method=c("quadratic", "linear", "loglinear"),
                         estimate.k=c("BIC", "BIC 3.4.12") )
{
  method <- match.arg(method)
  estimate.k <- match.arg(estimate.k)
  cluster.1d.dp(x, k, y, method, estimate.k, "L2Y",
                deparse(substitute(x)), deparse(substitute(y)))
}

library(testthat)
library(Ckmeans.1d.dp)
context("Ckmedian.1d.dp")

test_methods <- c("linear", "loglinear", "quadratic")

test_that("Unweighted k-median", {
  for(method in test_methods) {

    x <- c(-1, 2, 4, 5, 6)

    result <- Ckmedian.1d.dp(x, 3, method)
    expect_equal(result$size, c(1,1,3))
    expect_equal(result$cluster, c(1,2,3,3,3))
    expect_equal(result$centers, c(-1, 2, 5))
    expect_equal(result$withinss, c(0,0,2))

    x <- c(-.9, 1, 1.1, 1.9, 2, 2.05)
    result <- Ckmedian.1d.dp(x, c(1,6), method)
    expect_equal(result$size, c(1,2,3))
    expect_equal(result$centers, c(-0.9, 1, 2))
  }
})

library(testthat)
library(Ckmeans.1d.dp)
context("Cksegs.1d.dp")

test_methods <- c("quadratic")

test_that("k-segs", {
  for(method in test_methods) {

    y <- c(-1, 2, 4, 5, 6)

    result <- Cksegs.1d.dp(y, 3, method=method)
    expect_equal(result$size, c(1,1,3))
    expect_equal(result$cluster, c(1,2,3,3,3))
    expect_equal(result$centers, c(1, 2, 4))
    expect_equal(result$withinss, c(0,0,2))

    y <- c(-.9, 1, 1.1, 1.9, 2, 2.05)
    result <- Cksegs.1d.dp(y, c(1,6), method=method)
    expect_equal(result$size, c(1,2,3))
    expect_equal(result$centers, c(1, 2.5, 5))
  }
})

# test_Ckmeans.1d.dp.R
#
# Joe Song
# Created: May 3, 2016

library(testthat)
library(Ckmeans.1d.dp)
context("Ckmeans.1d.dp")

test_methods <- c("linear", "loglinear", "quadratic")

test_that("Weighted input", {

  for(method in test_methods) {

    x <- c(-1, 2, 4, 5, 6)
    y <- c( 4, 3, 1, 1, 1)

    result <- Ckmeans.1d.dp(x, 3, y, method)
    expect_equal(result$size, c(4,3,3))
    expect_equal(result$cluster, c(1,2,3,3,3))
    expect_equal(result$centers, c(-1, 2, 5))
    expect_equal(result$withinss, c(0,0,2))

    x <- c(-.9, 1, 1.1, 1.9, 2, 2.1)
    y <- c( 3,  1,   2,   2, 1, 1)
    result <- Ckmeans.1d.dp(x, c(1,6), y, method)
    expect_equal(result$size, c(3,3,4))
    expect_equal(result$centers, c(-0.9, (1+2.2)/3, (1.9*2+2+2.1)/4))

  }
})


test_that("Given the number of clusters", {

  for(method in test_methods) {

    x <- c(-1, 2, -1, 2, 4, 5, 6, -1, 2, -1)

    result <- Ckmeans.1d.dp(x, 3, method=method)
    expect_equal(result$size, c(4,3,3))

    cluster.truth <- c(1,2,1,2,3,3,3,1,2,1)
    expect_equal(result$cluster, cluster.truth)

    centers.truth <- c(c(-1, 2, 5))
    expect_equal(result$centers, centers.truth)
    withinss.truth <- c(0,0,2)
    expect_equal(result$withinss, withinss.truth)

    totss.truth <- sum(scale(x, scale=FALSE)^2)
    expect_equal(result$totss, totss.truth)
    expect_equal(result$tot.withinss, 2)
    expect_equal(result$betweenss, totss.truth - sum(withinss.truth))
  }
})

test_that("n<=k", {

  for(method in test_methods) {

    x <- c(3, 2, -5.4, 0.1);
    res <- Ckmeans.1d.dp(x, 4, method=method)

    cluster.truth <- c(4, 3, 1, 2)
    expect_equal(res$cluster, cluster.truth)
    centers.truth <- c(-5.4, 0.1, 2, 3)
    expect_equal(res$centers, centers.truth)
    withinss.truth <- c(0, 0, 0, 0)
    expect_equal(res$withinss, withinss.truth)
    size.truth <- c(1, 1, 1, 1)
    expect_equal(res$size, size.truth)

  }

})

test_that("k==2", {

  for(method in test_methods) {

    x <- 1:10
    res <- Ckmeans.1d.dp(x, 2, method=method)

    cluster.truth <- c(1, 1, 1, 1, 1, 2, 2, 2, 2, 2)
    expect_equal(res$cluster, cluster.truth)
    centers.truth <- c(3, 8)
    expect_equal(res$centers, centers.truth)
    withinss.truth <- c(10, 10)
    expect_equal(res$withinss, withinss.truth)
    size.truth <- c(5, 5)
    expect_equal(res$size, size.truth)
  }

})

test_that("k==1", {

  for(method in test_methods) {

    x <- c(-2.5, -2.5, -2.5, -2.5)
    res <- Ckmeans.1d.dp(x, 1, method=method)

    cluster.truth <- c(1, 1, 1, 1)
    expect_equal(res$cluster, cluster.truth)
    centers.truth <- c(-2.5)
    expect_equal(res$centers, centers.truth)
    withinss.truth <- c(0)
    expect_equal(res$withinss, withinss.truth)
    size.truth <- c(4)
    expect_equal(res$size, size.truth)

    x <- rep(1, 100)
    result <- Ckmeans.1d.dp(x, 1, method=method)
    expect_equal(result$size, 100)
  }

})

test_that("n==10, k==3", {

  for(method in test_methods) {

    x <- c(3, 3, 3, 3, 1, 1, 1, 2, 2, 2)
    res <- Ckmeans.1d.dp(x, 3, method=method)

    cluster.truth <- c(3, 3, 3, 3, 1, 1, 1, 2, 2, 2)
    expect_equal(res$cluster, cluster.truth)
    centers.truth <- c(1, 2, 3)
    expect_equal(res$centers, centers.truth)
    withinss.truth <- c(0, 0, 0)
    expect_equal(res$withinss, withinss.truth)
    size.truth <- c(3, 3, 4)
    expect_equal(res$size, size.truth)
  }
})


test_that("n==14, k==8", {

  for(method in test_methods) {

    x <- c(-3, 2.2, -6, 7, 9, 11, -6.3, 75, 82.6, 32.3, -9.5, 62.5, 7, 95.2)
    res <- Ckmeans.1d.dp(x, k=8, method=method)

    cluster.truth <- c(2, 2, 1, 3, 3, 3, 1, 6, 7, 4, 1, 5, 3, 8)
    expect_equal(res$cluster, cluster.truth)
    centers.truth <- c(-7.266666667, -0.400000000, 8.500000000, 32.300000000,
                       62.500000000, 75.000000000, 82.600000000, 95.200000000)
    expect_equal(res$centers, centers.truth)
    withinss.truth <- c(7.526666667, 13.520000000, 11.000000000, 0.000000000,
                        0.000000000, 0.000000000, 0.000000000, 0.000000000)
    expect_equal(res$withinss, withinss.truth)
    size.truth <- c(3, 2, 4, 1, 1, 1, 1, 1)
    expect_equal(res$size, size.truth)

  }

})

test_that("Estimating k example set 1", {

  for(method in test_methods) {

    x <- c(.9, 1, 1.1, 1.9, 2, 2.1)
    result <- Ckmeans.1d.dp(x, c(1,6), method=method)
    expect_equal(result$size, c(3,3))

    x <- rev(x)
    result <- Ckmeans.1d.dp(x, c(1,6), method=method)
    expect_equal(result$size, c(3,3))

    x <- 1:10
    result <- Ckmeans.1d.dp(x, k=c(1,10), method=method)
    expect_equal(result$size, 10) # before 3.4.12

    #if(method == "loglinear") {
    #  expect_equal(result$size, c(3,3,4))
    #} else {
    #  expect_equal(result$size, c(4,3,3))
    #}
  }

})

test_that("Estimating k example set 2", {

  for(method in test_methods) {

    x <- c(3.5, 3.6, 3.7, 3.1, 1.1, 0.9, 0.8, 2.2, 1.9, 2.1)
    res <- Ckmeans.1d.dp(x, k=c(2,5), method=method)

    cluster.truth <- c(3, 3, 3, 3, 1, 1, 1, 2, 2, 2)
    expect_equal(res$cluster, cluster.truth)
    centers.truth <- c(0.933333333333, 2.066666666667, 3.475000000000)
    expect_equal(res$centers, centers.truth)
    withinss.truth <- c(0.0466666666667, 0.0466666666667, 0.2075000000000)
    expect_equal(res$withinss, withinss.truth)
    size.truth <- c(3, 3, 4)
    expect_equal(res$size, size.truth)

  }

})

test_that("Estimating k example set 3 cosine", {

  for(method in test_methods) {

    x <- cos((-10:10))
    res <- Ckmeans.1d.dp(x, method=method)
    # format(res, digits=10)

    cluster.truth <- c(1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1)
    expect_equal(res$cluster, cluster.truth)
    centers.truth <- c(-0.6592474631, 0.6751193405)
    expect_equal(res$centers, centers.truth)
    withinss.truth <- c(1.0564793100, 0.6232976959)
    expect_equal(res$withinss, withinss.truth)
    size.truth <- c(12, 9)
    expect_equal(res$size, size.truth)

  }

})

test_that("Estimating k example set 4 gamma", {

  for(method in test_methods) {

    x <- dgamma(seq(1,10, by=0.5), shape=2, rate=1)
    res <- Ckmeans.1d.dp(x, method=method)
    # format(res, digits=10)

    cluster.truth <- c(3, 3, 3, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)
    expect_equal(res$cluster, cluster.truth)
    centers.truth <- c(0.01702193495, 0.15342151455, 0.32441508262)
    expect_equal(res$centers, centers.truth)
    withinss.truth <- c(0.006126754998, 0.004977009034, 0.004883305120)
    expect_equal(res$withinss, withinss.truth)
    size.truth <- c(13, 3, 3)
    expect_equal(res$size, size.truth)

  }

})

rm(test_methods)

# test_MC_WUC.R
#
# Hua Zhong
# Created: Aug 11, 2019
# Modified:
#   March 13, 2020. MS. Changed the random data from uniform integer
#     to standard normal to avoid examples with multiple optimal
#     solutions occurring to integers.
#
#   April 7, 2022. MS. Added "Test number-of-clusters selection".
#
#   July 25, 2025. MS. Set a random seed for
#     "Test number-of-clusters selection", due to a
#     ~.1% failure of testing reported by
#     Michael Chirico <chiricom@google.com>


library(testthat)
library(Ckmeans.1d.dp)
context("MDW_Ckmeans")

test_that("Test number-of-clusters selection", {

  set.seed(20260725) # Added July 25, 2026. MS.

  # Example 1.
  n <- c(20, 20, 20)
  x <- c(rnorm(n[1], mean=-6), rnorm(n[2], mean=0), rnorm(n[3], mean=6))
  ks <- 3
  res <- Ckmeans.1d.dp(x, k=ks)
  if(0) {
    plot(x, res$cluster)
    expect_equal(length(res$size), 3)
  }

  Y <- matrix(c(
    rep(c(1,0,0), times=n[1]),
    rep(c(0,1,0), times=n[2]),
    rep(c(0,0,1), times=n[3])
  ), byrow=TRUE, nrow=length(x))

  if(0) {
    o <- order(x)
    x <- x[o]
    Y <- Y[o, ]
  }

  res <- MultiChannel.WUC(x = x, y = Y, k = ks)

  plot(x, res$cluster)
  expect_equal(length(res$size), 3)

  # Example 2.
  n <- 100
  x <- rnorm(n)
  res <- Ckmeans.1d.dp(x, k=1:10)
  expect_equal(length(res$size), 1)

  Y <- t(sapply(x, function(val) {
    if(val > 0) {c(1,0)}
    else {c(0,1)}
  }))

  res <- MultiChannel.WUC(x = x, y = Y, k = 2) # 1:10)
  expect_equal(length(res$size), 2)
})

test_that("Test 1D weights comparing Ckmeans and MDW_Ckmeans", {
  total <- 10000

  i <- 0
  for(j in c(1:total)){

    # X <- sort(sample(x = c(1:100), size = 20, replace = TRUE)) # MS March 13, 2020
    # X <- sort(sample(x = rnorm(100), size = 20, replace = TRUE)) # MS March 13, 2020

    # MS April 9, 2022: It is no longer necessary to have X sorted:
    X <- sample(x = rnorm(100), size = 20, replace = TRUE)

    Y <- matrix(sample(x = c(1:100), size = 20, replace = TRUE),
                ncol=1, nrow=length(X))

    cluster.num <- c(5:10)

    res.1 <- MultiChannel.WUC(x = X, y = Y, k = cluster.num)

    res.2 <- suppressWarnings(
      Ckmeans.1d.dp(x = X, y = as.numeric(Y), k = cluster.num,
      method = "linear", estimate.k = "BIC"))

    # expect_equal(res.1$cluster, res.2$cluster)
    # expect_equal(res.1$centers, res.2$centers)
    # expect_equal(res.1$withinss, res.2$withinss)
    # expect_equal(res.1$size, res.2$size)
    # #expect_equal(res.1$BIC, res.2$BIC)

    if(all(c(all(res.1$cluster == res.2$cluster),
             all(res.1$centers - res.2$centers < 1e-13),
             all(res.1$withinss - res.2$withinss < 1e-13),
             all(res.1$size - res.2$size < 1e-13),
             all(res.1$BIC - res.2$BIC < 1e-13)
      ))) {
      i <- i + 1
    }else{
      # stop()
    }
  }
  # print(i)
  expect_equal((total - i) / total * 1000 , 0, tolerance = 1)

})

test_that("Test to compare linear and quadratic algorithms", {
  i <- 0
  total <- 10000
  for(j in c(1:total)){

    # MS April 9, 2022: It is no longer necessary to have X sorted:
    # Replay the following line
    # X <- sort(sample(x = c(1:100), size = 20, replace = TRUE))
    # by
    X <- sample(x = c(1:100), size = 20, replace = TRUE)

    Y <- matrix(sample(x = c(1:100), size = 20, replace = TRUE), ncol=1, nrow=length(X))

    cluster.num <- c(5:10)

    res.1 <- MultiChannel.WUC(x = X, y = Y, k = cluster.num)

    res.2 <- MultiChannel.WUC(x = X, y = Y, k = cluster.num)

    test <- all(all(res.1$cluster == res.2$cluster),
                all(res.1$centers == res.2$centers),
                all(res.1$withinss == res.2$withinss),
                all(res.1$size == res.2$size),
                all(res.1$BIC == res.2$BIC))
    i <- i + 1
  }
  # print(i)
  expect_equal((total - i) / total * 1000 , 0, tolerance = 1)

})

# test_MC_WUC.R
#
# Hua Zhong
# Created: Aug 11, 2019
# Modified:
#   March 13, 2020. MS. Changed the random data from uniform integer
#     to standard normal to avoid examples with multiple optimal
#     solutions occurring to integers.
#
#   April 7, 2022. MS. Added "Test number-of-clusters selection".

library(testthat)
library(Ckmeans.1d.dp)
context("MDW_Ckmeans")

test_that("Test number-of-clusters selection", {
  # Example 1.
  n <- c(20, 20, 20)
  x <- c(rnorm(n[1], mean=-6), rnorm(n[2], mean=0), rnorm(n[3], mean=6))
  ks <- 3
  res <- Ckmeans.1d.dp(x, k=ks)
  if(0) {
    plot(x, res$cluster)
    expect_equal(length(res$size), 3)
  }

  Y <- matrix(c(
    rep(c(1,0,0), times=n[1]),
    rep(c(0,1,0), times=n[2]),
    rep(c(0,0,1), times=n[3])
  ), byrow=TRUE, nrow=length(x))

  if(0) {
    o <- order(x)
    x <- x[o]
    Y <- Y[o, ]
  }

  res <- MultiChannel.WUC(x = x, y = Y, k = ks)

  plot(x, res$cluster)
  expect_equal(length(res$size), 3)

  # Example 2.
  n <- 100
  x <- rnorm(n)
  res <- Ckmeans.1d.dp(x, k=1:10)
  expect_equal(length(res$size), 1)

  Y <- t(sapply(x, function(val) {
    if(val > 0) {c(1,0)}
    else {c(0,1)}
  }))

  res <- MultiChannel.WUC(x = x, y = Y, k = 2) # 1:10)
  expect_equal(length(res$size), 2)
})

test_that("Test 1D weights comparing Ckmeans and MDW_Ckmeans", {
  total <- 10000

  i <- 0
  for(j in c(1:total)){

    # X <- sort(sample(x = c(1:100), size = 20, replace = TRUE)) # MS March 13, 2020
    # X <- sort(sample(x = rnorm(100), size = 20, replace = TRUE)) # MS March 13, 2020

    # MS April 9, 2022: It is no longer necessary to have X sorted:
    X <- sample(x = rnorm(100), size = 20, replace = TRUE)

    Y <- matrix(sample(x = c(1:100), size = 20, replace = TRUE),
                ncol=1, nrow=length(X))

    cluster.num <- c(5:10)

    res.1 <- MultiChannel.WUC(x = X, y = Y, k = cluster.num)

    res.2 <- suppressWarnings(
      Ckmeans.1d.dp(x = X, y = as.numeric(Y), k = cluster.num,
      method = "linear", estimate.k = "BIC"))

    # expect_equal(res.1$cluster, res.2$cluster)
    # expect_equal(res.1$centers, res.2$centers)
    # expect_equal(res.1$withinss, res.2$withinss)
    # expect_equal(res.1$size, res.2$size)
    # #expect_equal(res.1$BIC, res.2$BIC)

    if(all(c(all(res.1$cluster == res.2$cluster),
             all(res.1$centers - res.2$centers < 1e-13),
             all(res.1$withinss - res.2$withinss < 1e-13),
             all(res.1$size - res.2$size < 1e-13),
             all(res.1$BIC - res.2$BIC < 1e-13)
      ))) {
      i <- i + 1
    }else{
      # stop()
    }
  }
  # print(i)
  expect_equal((total - i) / total * 1000 , 0, tolerance = 1)

})

test_that("Test to compare linear and quadratic algorithms", {
  i <- 0
  total <- 10000
  for(j in c(1:total)){

    # MS April 9, 2022: It is no longer necessary to have X sorted:
    # Replay the following line
    # X <- sort(sample(x = c(1:100), size = 20, replace = TRUE))
    # by
    X <- sample(x = c(1:100), size = 20, replace = TRUE)

    Y <- matrix(sample(x = c(1:100), size = 20, replace = TRUE), ncol=1, nrow=length(X))

    cluster.num <- c(5:10)

    res.1 <- MultiChannel.WUC(x = X, y = Y, k = cluster.num)

    res.2 <- MultiChannel.WUC(x = X, y = Y, k = cluster.num)

    test <- all(all(res.1$cluster == res.2$cluster),
                all(res.1$centers == res.2$centers),
                all(res.1$withinss == res.2$withinss),
                all(res.1$size == res.2$size),
                all(res.1$BIC == res.2$BIC))
    i <- i + 1
  }
  # print(i)
  expect_equal((total - i) / total * 1000 , 0, tolerance = 1)

})

# visualize.R -- visualization functions for Ckmeans.1d.dp
#
# Joe Song
# Created: Oct 1, 2016
# Modified:
#   Oct 12, 2016. Revised ahist().
#   Oct 16, 2016. Moved ahist() function to a new R file ahist.R
#   May 29, 2017. Added two functions plot.Ckmedian.1d.dp() and
#     plot.Cksegs.1d.dp
#   Nov 4, 2020. Changed the plot type to histogram like

plotBIC <-
  function(ck, xlab="Number of clusters k",
           ylab = "BIC/n", type="b",
           sub=paste("n =", length(ck$cluster)),
           main=paste("Bayesian information criterion",
                      "(normalized by sample size)", sep="\n"),
           ...)
  {
    if(length(ck$BIC)>0) {
      n <- length(ck$cluster)
      k <- as.numeric(sub("k=", "", names(ck$BIC)))
      graphics::plot(k, ck$BIC/n, xlab=xlab, ylab=ylab,
                     type=type, sub=sub, main=main, ...)

      if(length(ck$BIC)>1) {
        kstar <- length(ck$size)
        graphics::abline(v=kstar, lty="dashed")
        if(kstar <= mean(k)) {
          graphics::text(kstar, min(ck$BIC)/n,
                         paste0(" k*=", kstar), font=2, adj=0)
        } else {
          graphics::text(kstar, min(ck$BIC)/n,
                         paste0("k*=", kstar, " "), font=2, adj=1)
        }
      }
    }
    invisible(ck)
  }

plot.Ckmeans.1d.dp <-
  function(x, xlab=NULL, ylab=NULL, main=NULL,
           sub=NULL, col.clusters=NULL, ...)
    # function(ck, xlab=ck$xname,
    #          ylab=ifelse(ck$yname=="1", "Weight", ck$yname),
    #          main=paste("Optimal k-means clustering of", ck$xname),
    #          sub=paste("n =", length(ck$cluster)),
    #          col.clusters=seq_along(ck$size),
    #          ...)
  {
    ck <- x
    if(is.null(xlab)) xlab <- ck$xname
    if(is.null(ylab)) ylab <- ifelse(ck$yname=="1", "Weight", ck$yname)
    if(is.null(main)) main <- paste("Optimal",
                                    ifelse(ck$yname=="1", "univariate",
                                           "weighted univariate"),
                                    "k-means clustering of", ck$xname)
    if(is.null(sub)) sub=paste("n =", length(ck$cluster))
    if(is.null(col.clusters)) col.clusters <- seq_along(ck$size)

    if(exists(ck$xname, mode="numeric")) {
      x <- get(ck$xname, mode="numeric")
    } else {
      x <- eval(parse(text=ck$xname))
    }

    if(exists(ck$yname, mode="numeric")) {
      y <- get(ck$yname, mode="numeric")
    } else {
      y <- eval(parse(text=ck$yname))
    }

    if(length(y) == 1) {
      y <- rep(y, length(x))
    }

    graphics::plot(x, y, type="h",
                   xlab=xlab, ylab=ylab, main=main, sub=sub,
                   col=col.clusters[ck$cluster],
                   ...)

    # ks <- seq_along(ck$size)
    # sapply(ks, function(q) {
    #   graphics::segments(x[ck$cluster == q], 0,
    #                      x[ck$cluster == q], y[ck$cluster == q],
    #                      col=col.clusters[q], ...) } )
    # }

    invisible(ck)
  }


plot.Ckmedian.1d.dp <-function(x, xlab=NULL, ylab=NULL, main=NULL,
                               sub=NULL, col.clusters=NULL, ...)
{
  ck <- x
  if(is.null(main)) {
    main <- paste("Optimal",
                  ifelse(ck$yname=="1", "univariate",
                         "weighted univariate"),
                  "k-median clustering of", ck$xname)
  }

  plot.Ckmeans.1d.dp(ck, xlab, ylab, main=main, sub, col.clusters, ...)
}

plot.Cksegs.1d.dp <- function(x, xlab=NULL, ylab=NULL, main=NULL,
                              sub=NULL, col.clusters=NULL, ...)
{
  ck <- x
  if(is.null(xlab)) xlab <- ck$xname
  if(is.null(ylab)) ylab <- ifelse(ck$yname=="1", "Weight", ck$yname)
  if(is.null(main)) main <- paste("Optimal segmentation of", ck$xname, "by", ck$yname,
                                  "\n(Piece-wise constant approximation)")
  if(is.null(sub)) sub=paste("n =", length(ck$cluster))
  if(is.null(col.clusters)) col.clusters <- seq_along(ck$size)

  if(exists(ck$xname, mode="numeric")) {
    x <- get(ck$xname, mode="numeric")
  } else {
    x <- eval(parse(text=ck$xname))
  }

  if(exists(ck$yname, mode="numeric")) {
    y <- get(ck$yname, mode="numeric")
  } else {
    y <- eval(parse(text=ck$yname))
  }

  if(length(y) == 1) {
    y <- rep(y, length(x))
  }

  graphics::plot(x, y, type="h",
                 xlab=xlab, ylab=ylab, main=main, sub=sub,
                 col=col.clusters[ck$cluster],
                 ...)

  #  ks <- seq_along(ck$size)
  #  sapply(ks, function(q) {
  #  graphics::segments(min(x[ck$cluster == q]), mean(y[ck$cluster == q]),
  #                     max(x[ck$cluster == q]), mean(y[ck$cluster == q]),
  #                     col=col.clusters[q], ...) } )

  invisible(ck)
}

# print.R -- define S3 print functions for Ckmeans.1d.dp,
#            Ckmedian.1d.dp, Cksegs.1d.dp
#
# Joe Song
#
# Created: May 29, 2017.

## print method for Ckmeans.1d.dp object
print.Ckmeans.1d.dp <- function(x, ...)
{
  with(x, {
    cat("\nCluster centers:\n")
    print(centers, ...)
    cat("\nCluster index associated with each element:\n")
    print(cluster, ...)
    cat("\nWithin-cluster sum of squares:\n")
    print(withinss, ...)
    if(length(size) > 1) {
      cat("Ckmeans.1d.dp returns", length(size), "optimal clusters of sizes",
          paste(size, collapse=", "), "\n")
      cat("  minimum total within-cluster sum of squares:", tot.withinss, "\n")
      cat("  maximum between-cluster sum of squares:", betweenss, "\n")
      cat("  total sum of squares of input vector:", totss, "\n")
      if(totss > 0) {
        cat("  maximum (between-SS / total-SS):",
            round(1000 * betweenss / totss)/10, "%\n")
      }
    } else {
      cat("Ckmeans.1d.dp returns one cluster containing all elements.\n")
    }
  })

  cat("\nAvailable components:\n")
  print(names(x))
  invisible(x)
}

print.Ckmedian.1d.dp <- function(x, ...)
{
  with(x, {
    cat("\nCluster centers:\n")
    print(centers, ...)
    cat("\nCluster index associated with each element:\n")
    print(cluster, ...)
    cat("\nWithin-cluster sum of L1 distances:\n")
    print(withinss, ...)
    if(length(size) > 1) {
      cat("Ckmedian.1d.dp returns", length(size), "optimal clusters of sizes",
          paste(size, collapse=", "), "\n")
      #cat("  minimum total within-cluster sum of squares:", tot.withinss, "\n")
      #cat("  maximum between-cluster sum of squares:", betweenss, "\n")
      #cat("  total sum of squares of input vector:", totss, "\n")
      #if(totss > 0) {
      #  cat("  maximum (between-SS / total-SS):",
      #      round(1000 * betweenss / totss)/10, "%\n")
      #}
    } else {
      cat("Ckmedian.1d.dp returns one cluster containing all elements.\n")
    }
  })

  cat("\nAvailable components:\n")
  print(names(x))
  invisible(x)
}

print.Cksegs.1d.dp <- function(x, ...)
{
  with(x, {
    cat("\nSegment centers:\n")
    print(centers, ...)
    cat("\nSegment index associated with each element:\n")
    print(cluster, ...)
    cat("\nWithin-segment sum of L2 distances in Y:\n")
    print(withinss, ...)
    if(length(size) > 1) {
      cat("Cksegs.1d.dp returns", length(size), "optimal segments of sizes",
          paste(size, collapse=", "), "\n")
      #cat("  minimum total within-cluster sum of squares:", tot.withinss, "\n")
      #cat("  maximum between-cluster sum of squares:", betweenss, "\n")
      #cat("  total sum of squares of input vector:", totss, "\n")
      #if(totss > 0) {
      #  cat("  maximum (between-SS / total-SS):",
      #      round(1000 * betweenss / totss)/10, "%\n")
      #}
    } else {
      cat("Cksegs.1d.dp returns one segment containing all elements.\n")
    }
  })

  cat("\nAvailable components:\n")
  print(names(x))
  invisible(x)
}

MultiChannel.WUC <- function(x, y,  k=c(1,9)){
  result <- MCW_main(x = x, y = y, Kmin = min(k), Kmax = max(k),
                     estimate_k = "BIC", method = "linear")

  if(length(result$cluster) > 0) {
    k.opt <- max(result$cluster)
  } else {
    k.opt <- 0
  }

  if(nrow(y) == length(x) && all(colSums(y) != 0)) {
    totss <- sum(apply(y, 2, function(y.tmp){
      return(sum(y.tmp * (x - sum(as.numeric(x * y.tmp)) / sum(y.tmp))^2))
    }))
  } else {
    # totss <- sum(scale(x, scale=FALSE)^2) scale function is VERY SLOW!
    totss <- sum((x - sum(as.numeric(x)) / length(x))^2)
  }

  tot.withinss <- sum(result$withinss[1:k.opt])
  betweenss <- totss - tot.withinss
  BIC <- result$BIC
  names(BIC) <- paste0("k=", k)

  r <- structure(
    list(
      cluster = result$cluster, centers = result$centers[1:k.opt],
      withinss = result$withinss[1:k.opt], size = result$size[1:k.opt],
      totss = totss, tot.withinss = tot.withinss, betweenss = betweenss,
      BIC = BIC, xname = deparse(substitute(x)), yname = deparse(substitute(y))
    ),
    class = "MultiChannelClusters")

  return( r )
}

plot.MultiChannelClusters <- function(
  x, xlab=NULL, ylab=NULL, main=NULL,
  sub=NULL, col.clusters=NULL, ...)
{
  ck <- x
  if(is.null(xlab)) xlab <- ck$xname
  if(is.null(ylab)) ylab <- ifelse(ck$yname=="1", "Weight", ck$yname)
  if(is.null(main)) main <-
    "Optimal multichannel weighted clustering"
  # if(is.null(sub)) sub=paste("n =", length(ck$cluster))
  if(is.null(col.clusters)) col.clusters <- seq_along(ck$size)

  if(exists(ck$xname, mode="numeric")) {
    x <- get(ck$xname, mode="numeric")
  } else {
    x <- eval(parse(text=ck$xname))
  }

  if(exists(ck$yname, mode="numeric")) {
    y <- get(ck$yname, mode="numeric")
  } else {
    y <- eval(parse(text=ck$yname))
  }

  graphics::split.screen(c(ncol(y), 1))

  for(ch in seq_len(ncol(y))) {
    graphics::screen(ch)
    main.text <- paste0(
      ifelse(ch == 1, main, ""),
      "\nChannel ", ch, ",  weight = ", colSums(y)[ch])

    graphics::plot(
      x, y[, ch], type="h", xlab=xlab, ylab=ylab,
      main=main.text, # sub=sub,
      col=col.clusters[ck$cluster],
      ...)
  }
  graphics::close.screen(all.screens = TRUE)
  invisible(ck)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

Ckmeans_1d_dp <- function(x, length, y, ylength, minK, maxK, cluster, centers, withinss, size, BICs, estimate_k, method) {
    .Call(`_Ckmeans_1d_dp_Ckmeans_1d_dp`, x, length, y, ylength, minK, maxK, cluster, centers, withinss, size, BICs, estimate_k, method)
}

Ckmedian_1d_dp <- function(x, length, y, ylength, minK, maxK, cluster, centers, withinss, size, BICs, estimate_k, method) {
    .Call(`_Ckmeans_1d_dp_Ckmedian_1d_dp`, x, length, y, ylength, minK, maxK, cluster, centers, withinss, size, BICs, estimate_k, method)
}

Cksegs_1d_dp <- function(x, length, y, ylength, minK, maxK, cluster, centers, withinss, size, BICs, estimate_k, method) {
    .Call(`_Ckmeans_1d_dp_Cksegs_1d_dp`, x, length, y, ylength, minK, maxK, cluster, centers, withinss, size, BICs, estimate_k, method)
}

MCW_main <- function(x, y, Kmin, Kmax, estimate_k, method) {
    .Call(`_Ckmeans_1d_dp_MCW_main`, x, y, Kmin, Kmax, estimate_k, method)
}

# ahist.R -- Adaptive histograms using univariate k-means
#
# Joe Song
# Created: Oct 16, 2016. Extracted from visualize.R
# Modified:
#   Oct 12, 2016. Revised ahist().
#   December 6, 2016.
#     1. Added weighted option to ahist() function.
#     2. Broke ahist() to multiple functions

discontinuous.breaks <- function(xs, size)
{ # xs: sorted data
  # size: a vector for the number of points in each cluster

  k <- length(size)

  breaks <- vector("double", length=2*k)

  i <- 1
  for(q in 1:k) {

    # estimate the lower and upper boundaries
    #   cluster q

    j <- i+size[q]-1

    if(size[q] == 1 || stats::var(xs[i:j]) == 0) {
      if(k == 1) {
        # there exists a single cluster of identical elements
        breaks <- 1
        break
      } else {
        # distance to the left (lower) cluster:
        dl <- ifelse(q==1, Inf, xs[i] - xs[i-1])
        # distance to the right (upper) cluster:
        du <- ifelse(q==k, Inf, xs[j+1] - xs[j])
        d <- min(c(dl, du)) / 6
        lb <- xs[i] - d
        ub <- xs[j] + d
      }
    } else {
      # Max difference between consecutive points
      #   within cluster q:
      delta <- max(diff(xs[i:j]))

      lb <- xs[i] - delta
      lmid <- ifelse(q == 1, -Inf, (xs[i-1] + xs[i]) / 2.0)
      lb <- max(c(lb, lmid))

      ub <- xs[j] + delta
      umid <- ifelse(q == k, Inf, (xs[j] + xs[j+1]) / 2.0)
      ub <- min(c(ub, umid))
    }

    breaks[2*q-1] <- lb
    breaks[2*q] <- ub

    i <- i + size[q]
  }

  breaks <- unique(breaks)

  return(breaks)
}

midpoint.breaks <- function(xs, size)
{
  # xs: sorted data
  # size: a vector for the number of points in each cluster

  k <- length(size)

  breaks <- vector("double", length=k+1)

  if(k > 1) {
    i <- size[1]
    for(q in 2:k) {
      breaks[q] <- (xs[i] + xs[i+1])/2
      i <- i + size[q]
    }
  }

  breaks[1] <- xs[1]
  breaks[k+1] <- xs[length(xs)]
  return(breaks)
}

plot.ahist <- function(
  h, x, weight, xlab, wlab, main, col, border, lwd,
  col.stick, lwd.stick, add.sticks,
  skip.empty.bin.color, ...)
{
  opar <- graphics::par(lwd=lwd, mar=c(5,4,4,4)+0.1)
  if(skip.empty.bin.color
     && !is.null(col) && length(col) > 1
     && min(h$counts) == 0) {
    # adjust color for empty bins
    colors <- vector(length = length(h$counts))
    j <- 1
    for(i in seq_along(colors)) {
      colors[i] <- col[(j-1) %% length(col) + 1]
      if(h$counts[i] > 0) {
        j <- j + 1
      }
    }
  } else {
    colors <- col
  }

  graphics::plot(h, main=main, xlab=xlab, col=colors,
                 border=border, ...)

  if(add.sticks) {
    if(length(weight) == 1) {
      # equal weights
      if(h$equidist) {
        graphics::segments(x, -max(h$counts)/10, x, 0,
                           col=col.stick, lwd=lwd.stick)
      } else {
        graphics::segments(x, -max(h$density)/10, x, 0,
                           col=col.stick, lwd=lwd.stick)
      }
    } else {
      # unequal weights

      graphics::mtext(wlab, side = 4, line = 2.75,
                      cex=graphics::par("cex") * graphics::par("cex.lab"), ...)

      graphics::par(new=T)

      graphics::plot(x, weight, type="h", axes=FALSE,
                     xlab=NA, ylab=NA,
                     col=col.stick, lwd=lwd.stick)
      graphics::axis(4)

      mar <- graphics::par("mar")
      mar.legend <- c(mar[1], mar[2]-4, mar[3]-2, mar[4]-2)
      mar.legend <- ifelse(mar.legend >= 0, mar.legend, 0)
      graphics::par(new=T, mar=mar.legend)
      graphics::plot.new()

      ylab <- ifelse(h$equidist, "Frequency", "Density")

      if(0) {
        graphics::legend("bottomleft", c(ylab, wlab), horiz = TRUE,
                         pch=c(22, NA), lty=c(NA, 1),
                         bg="transparent",
                         col=c(border, col.stick),
                         pt.bg=c(colors[1], NA),
                         lwd=c(NA, lwd.stick)
        )
      } else {
        graphics::legend("topleft", c(ylab), bty="n",
                         pch=c(22), lty=c(NA),
                         bg="transparent",
                         col=c(border),
                         pt.bg=c(colors[1]), pt.cex=2, pt.lwd = lwd,
                         lwd=c(NA)
        )
        graphics::legend("topright", c(wlab), bty="n",
                         pch=c(NA), lty=c(1),
                         bg="transparent",
                         col=c(col.stick),
                         pt.bg=c(NA),
                         lwd=c(ifelse(lwd.stick >= 1, lwd.stick, 1))
        )

      }
    }
  }

  graphics::par(opar)
  invisible(h)
}

ahist <- function(
  x, k = c(1,9), breaks=NULL, data=NULL, weight=1,
  plot = TRUE, xlab = deparse(substitute(x)),
  wlab = deparse(substitute(weight)),
  main = NULL, col = "lavender", border = graphics::par("fg"),
  lwd = graphics::par("lwd"),
  col.stick = "gray", lwd.stick = 1, add.sticks=TRUE,
  style = c("discontinuous", "midpoints"),
  skip.empty.bin.color=TRUE,
  ...)
  # adaptive histogram
{
  # x or data may be unsorted, which the return value of the function
  #   must be based on

  style <- match.arg(style)

  xname <- deparse(substitute(x))

  if(is.null(main)) {
    main <- paste(ifelse(is.null(breaks),
                         "Adaptive histogram", "Histogram"),
                  "of", xname)
  }

  if(mode(x) == "numeric") {

    if(length(x) == 0) {
      warning(paste0("\'", xname, "\'", " is empty!"))
      return()
    }

    o <- order(x)
    xs <- x[o]     # xs <- sort(x)

    if(length(weight) > 1) {
      r <- Ckmeans.1d.dp(xs, k=k, y=weight[o])
    } else {
      r <- Ckmeans.1d.dp(xs, k=k)
    }

  } else if(inherits(x, "Ckmeans.1d.dp")) {
    r <- x
    xname <- r$xname
    x <- data
    if(is.null(data)) {
      stop("data argument must be specified when the class of x is Ckmeans.1d.dp!\n")
    }

    o <- order(data)
    xs <- data[o] # xs <- sort(data)
  } else {
    warning(paste0("\'", xname, "\'", " must be numeric or of class Ckmeans.1d.dp!"))
    return()
  }

  if(is.null(breaks)) {

    if(length(weight) > 1) { # r$size is weighted counts of each cluster.
      # compute actual count of points in each cluster:
      counts <- table(r$cluster[o])
    } else { # in case the cluster is not numbered increasingly with xs
      counts <- r$size
    }

    if(style=="midpoints") {

      breaks <- midpoint.breaks(xs, counts)

    } else if(style=="discontinuous") {

      breaks <- discontinuous.breaks(xs, counts)

    }
  }

  h <- graphics::hist(x, breaks=breaks, plot=FALSE,
                      warn.unused=FALSE, ...)

  if(length(weight) > 1) { # compute weighted count and density

    h$counts <- rep(0, length(breaks)-1)

    table <- stats::aggregate(weight,
                              by=list(bin=cut(x, breaks, labels=FALSE)),
                              FUN=sum) # weighted counts

    h$counts[table[,"bin"]] <- table[,2]

    h$density <- h$counts / diff(breaks) / sum(h$counts) # weighted density
  }

  h$xname <- xname

  if(plot) {
    plot.ahist(h, x, weight, xlab, wlab, main, col, border,
               lwd, col.stick, lwd.stick, add.sticks,
               skip.empty.bin.color, ...)
  } else {
    return(h)
  }
}

---
title: "Note: Weight scaling in cluster analysis"
author: "Joe Song"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Note: Weight scaling in cluster analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

The function ```Ckmeans.1d.dp()``` can perform optimal weighted univariate clustering. Depending on the application, weights can indicate sample size, certainty, or signal intensity. Relative values of weights are always consequential on cluster results. Absolute values of weights have an impact on the number of clusters when it must be estimated.

### The linear scale of weights is consequential when estimating the number of clusters

When the number of clusters must be estimated, the linear scale of weights heavily influences the estimated number of clusters $k$. The reason is that linear scaling has a nonlinear effect when calculating the Bayesian information criterion. A large scale will promote more clusters to be used.

Here is a guideline on how to scale the weights:

* If weights are the numbers of repeated observations at each data point, they should be used as is and not linearly scaled.

* If the weights are not related to sample size but are some measure of emphasis, they should be scaled to sum up to the observed sample size of the entire data set.

* If the weights sum up to one, it implies that the sample size of the data set is one. In this case, the number of clusters may be severely underestimated.

### Linear weight scaling is uninfluential when the number of clusters is given

When an exact number of clusters $k$ is given by the user, linear weight scaling does not influence cluster analysis in theory. The clustering results are expected to be identical for any linear scaling of weights. However, a large numerical weight can cause overflow and thus should be linearly scaled down to a more tractable range.

---
title: "Tutorial: Optimal univariate clustering"
author: "Joe Song and Haizhou Wang"
date: "Updated 2020-11-07"
output: 
 rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{Tutorial: Optimal univariate clustering}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Here, we illustrate optimal univariate clustering function ```Ckmeans.1d.dp```. It uses a given number of clusters $k$, or estimates $k$ if a range is provided. If $k$ is unspecified, a default range from 1 to 9 is used. It can also perform optimal weighted clustering when a non-negative weight vector is specified as input. Weighted clustering can be used to analyze signals such as time series data, spectral data, genetic or epigenetic events along a chromosome.

```{r, message=FALSE}
require(Ckmeans.1d.dp)
require(RColorBrewer)
```

## 1. Cluster univariate data given the number of clusters $k$

### Example 1.

Cluster data generated from a Gaussian mixture model of three components.

The number of clusters is provided.
```{r, fig.height=4, fig.width=7}
x <- c(rnorm(50, sd=0.3), rnorm(50, mean=1, sd=0.3), rnorm(50, mean=2, sd=0.3))
k <- 3 # Divide x into 3 clusters
result <- Ckmeans.1d.dp(x, k)
colors <- brewer.pal(k, "Dark2")
plot(result, col.clusters = colors)
plot(x, col=colors[result$cluster], pch=result$cluster, cex=1.5,
     main="Optimal univariate clustering given k",
     sub=paste("Number of clusters given:", k))
abline(h=result$centers, col=colors, lty="dashed", lwd=2)
legend("bottomright", paste("Cluster", 1:k), col=colors, pch=1:k, cex=1, bty="n")
```

## 2. Cluster univariate data with an estimated number of clusters $k$

### Example 2.

Cluster data generated from a Gaussian mixture model of three components. The number of clusters is determined by Bayesian information criterion:
```{r, fig.height=4, fig.width=7}
x <- c(rnorm(50, mean=-1, sd=0.3), rnorm(50, mean=1, sd=1), rnorm(50, mean=2, sd=0.4))
# Divide x into k clusters, k automatically selected (default: 1~9)
result <- Ckmeans.1d.dp(x)
k <- max(result$cluster)
colors <- brewer.pal(k, "Dark2")
plot(result, col.clusters = colors)
plot(x, col=colors[result$cluster], pch=result$cluster, cex=1.5,
     main="Optimal univariate clustering with k estimated",
     sub=paste("Number of clusters is estimated to be", k))
abline(h=result$centers, col=colors, lty="dashed", lwd=2)
legend("topleft", paste("Cluster", 1:k), col=colors, pch=1:k, cex=1, bty="n")
```

## 3. Optimal weighted univariate clustering for time series analysis

### Example 3.

We segment a time course to identify peaks using weighted clustering. The input data is the time stamp of obtaining each intensity measurement; the weight is the signal intensity.

```{r, fig.height=4, fig.width=7}
n <- 160
t <- seq(0, 2*pi*2, length=n)
n1 <- 1:(n/2)
n2 <- (max(n1)+1):n
y1 <- abs(sin(1.5*t[n1]) + 0.1*rnorm(length(n1)))
y2 <- abs(sin(0.5*t[n2]) + 0.1*rnorm(length(n2)))
y <- c(y1, y2)

w <- y^8 # stress the peaks 
res <- Ckmeans.1d.dp(t, k=c(1:10), w)
k <- max(res$cluster)
colors <- brewer.pal(k, "Set1")
plot(res, col.clusters = colors)
grid()
plot(t, w, main = "Time course clustering / peak calling", 
     col=colors[res$cluster], pch=res$cluster, type="h", 
     xlab="Time t", ylab="Transformed intensity w")
grid()
abline(v=res$centers, col="gray", lty="dashed")
text(res$centers, max(w) * .95, cex=0.75, font=2,
     paste(round(res$size / sum(res$size) * 100), "/ 100"))
```

## 4. Obtaining midpoints between consecutive clusters

It is often desirable to visualize boundaries between consecutive clusters. The `ahist()` function offers several ways to estimate cluster boundaries. The simplest is to use the midpoint between the two closest points in two consecutive clusters, as illustrated in the code below.  
```{r, fig.height=4, fig.width=7}
x <- c(7, 4, 1, 8, 15, 22, -1)
k <- 3
ckm <- Ckmeans.1d.dp(x, k=k)
midpoints <- ahist(ckm, style="midpoints", data=x, plot=FALSE)$breaks[2:k]

colors <- brewer.pal(k, "Set2")
plot(ckm, col.clusters = colors, lwd=5,
     main="Midpoints as cluster boundaries")
abline(v=midpoints, col="RoyalBlue", lwd=3, lty=2)
legend("topright", "Midpoints", lwd=3, lty=2, col="RoyalBlue")
```

---
title: "Tutorial: Adaptive versus regular histograms"
author: "Joe Song"
date: "Updated 2020-11-07"
output: 
   rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{Tutorial: Adaptive versus regular histograms}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Adaptive histograms can reveal patterns in data more effectively than regular (equal-bin-width) histograms. The "discontinuous" style adaptive histogram is recommended because it adapts to the data without attempting to assign a non-zero density to every bin. When bins are not equal with, the vertical axis of the histogram is density instead of frequencies.

## Example 1: Data from a Gaussian mixture model using one bin per cluster

Plot an adaptive histogram from data generated by
   a Gaussian mixture model with three components:
```{r, fig.width=6}
require("Ckmeans.1d.dp")
x <- c(rnorm(40, mean=-2, sd=0.3),
       rnorm(45, mean=1, sd=0.1),
       rnorm(70, mean=3, sd=0.2))
ahist(x, col="lightblue", sub=paste("n =", length(x)),
      col.stick="darkblue", lwd=2, xlim=c(-4,4),
      main="Example 1. Gaussian mixture model with 3 components\n(one bin per component)\nAdaptive histogram")
```

When breaks is specified, ahist will call hist (regular histogram function in R).
```{r, fig.width=6}
ahist(x, breaks=3, col="lightgreen", sub=paste("n =", length(x)),
      col.stick="forestgreen", lwd=2,
      main="Example 1. Regular histogram")
```

## Example 2: Data from a Gaussian mixture model using three bins per cluster

Plot an adaptive histogram from data generated by
   a Gaussian mixture model with three components using a given
   number of bins
```{r, fig.width=6}
ahist(x, k=9, col="lavender", col.stick="navy",
      sub=paste("n =", length(x)), lwd=2,
      main="Example 2. Gaussian mixture model with 3 components\n(on average 3 bins per component)\nAdaptive histogram")
```

When breaks is specified, ahist will call hist (regular histogram function in R).
```{r, fig.width=6}
ahist(x, breaks=9, col="lightgreen", col.stick="forestgreen",
      sub=paste("n =", length(x)), lwd=2,
      main="Example 2. Regular histogram")
```

## Example 3: Adaptive histogram of protein DNase

The DNase data frame has 176 rows and 3 columns of
   data obtained during development of an ELISA assay for the
   recombinant protein DNase in rat serum:
```{r, fig.show='hold', fig.width=6}
data(DNase)
res <- Ckmeans.1d.dp(DNase$density)
kopt <- length(res$size)
ahist(res, data=DNase$density, col=rainbow(kopt), col.stick=rainbow(kopt)[res$cluster],
      sub=paste("n =", length(x)), border="transparent",
      xlab="Optical density of protein DNase",
      main="Example 3. Elisa assay of DNase in rat serum\nAdaptive histogram")
```

Using the same data with Example 3, this example demonstrates the inadequacy of equal-bin-width histograms. The third bin gives a false sense of sample distribution.

We can specifiy breaks="Sturges" in ahist() function to use equal-bin-width histograms. The difference is that sticks are added to the histogram by ahist(), but not by the R provided hist() function.

```{r, fig.show='hold', fig.width=6}
ahist(DNase$density, breaks="Sturges", col="palegreen",
      add.sticks=TRUE, col.stick="darkgreen",
      main="Example 3. Elisa assay of DNase in rat serum\nRegular histogram (equal bin width)",
      xlab="Optical density of protein DNase")
```

## Example 4. Repetitive data

Cluster data with repetitive elements:
```{r, fig.show='hold', fig.width=6}
x <- c(1,1,1,1, 3,4,4, 6,6,6)
ahist(x, k=c(2,4), col="gray",
      lwd=2, lwd.stick=6, col.stick="chocolate",
      main="Example 4. Adaptive histogram of repetitive elements")
ahist(x, breaks=3, col="lightgreen",
      lwd=2, lwd.stick=6, col.stick="forestgreen",
      main="Example 4. Regular histogram")

```

\name{Univariate Clustering}
\alias{Univariate Clustering}
\alias{Ckmeans.1d.dp}
\alias{Ckmedian.1d.dp}

\title{Optimal (Weighted) Univariate Clustering}

\description{
Perform optimal univariate \eqn{k}{k}-means or \eqn{k}{k}-median clustering in linear (fastest), loglinear, or quadratic (slowest) time.
}

\usage{
Ckmeans.1d.dp(x, k=c(1,9), y=1,
              method=c("linear", "loglinear", "quadratic"),
              estimate.k=c("BIC", "BIC 3.4.12"))

Ckmedian.1d.dp(x, k=c(1,9), y=1,
               method=c("linear", "loglinear", "quadratic"),
               estimate.k=c("BIC", "BIC 3.4.12"))
}

\arguments{
  \item{x}{a numeric vector of data to be clustered. All \code{NA} elements must be removed from \code{x} before calling this function. The function will run faster on sorted \code{x} (in non-decreasing order) than an unsorted input.}

  \item{k}{either an exact integer number of clusters, or a vector of length two specifying the minimum and maximum numbers of clusters to be examined. The default is \code{c(1,9)}. When \code{k} is a range, the actual number of clusters is determined by Bayesian information criterion.}

  \item{y}{a value of 1 (default) to specify equal weights of 1 for each element in \code{x}, or a numeric vector of unequal non-negative weights for each element in \code{x}. It is highly recommended to use positive (instead of zero) weights to account for the influence of every element. The weights have a strong impact on the clustering result. When the number of clusters \code{k} is given as a range, the weights should be linearly scaled to sum up to the observed sample size. Currently, \code{Ckmedian.1d.dp} only works with an equal weight of 1.}

  \item{method}{a character string to specify the speedup method to the original cubic runtime dynamic programming. The default is \code{"linear"}. All methods generate the same optimal results but differ in runtime or memory usage. See Details.}

  \item{estimate.k}{a character string to specify the method to estimate optimal \code{k}. This argument is effective only when a range for \code{k} is provided. The default is \code{"BIC"}. See Details.}

}

\author{
	Joe Song and Haizhou Wang
}

\details{
\code{Ckmean.1d.dp} minimizes unweighted or weighted within-cluster sum of squared distance (L2).

\code{Ckmedian.1d.dp} minimizes within-cluster sum of distance (L1). Only unweighted solution is implemented and guarantees optimality.

In contrast to the heuristic \var{k}-means algorithms
implemented in function \code{\link{kmeans}}, this function
optimally assigns elements in numeric vector \code{x}
into \code{k} clusters by dynamic programming
\insertCite{Wang2011Ckmeans,song2020wuc}{Ckmeans.1d.dp}.
It minimizes the total of within-cluster sums of
squared distances (\var{withinss}) between each element
and its corresponding cluster mean. When a range is
provided for \code{k}, the exact number of clusters
is determined by Bayesian information criterion
\insertCite{song2020wuc}{Ckmeans.1d.dp}. Different from
the heuristic \var{k}-means algorithms whose results
may be non-optimal or change from run to run, the
result of Ckmeans.1d.dp is guaranteed to be optimal
and reproducible, and its advantage in efficiency and
accuracy over heuristic \var{k}-means methods is most
pronounced at large \var{k}.

The \code{estimate.k} argument specifies the method to select optimal \code{k} based on the Gaussian mixture model using the Bayesian information criterion (BIC). When \code{estimate.k="BIC"}, it effectively deals with variance estimation for a cluster with identical values. When \code{estimate.k="BIC 3.4.12"}, it uses the code in version 3.4.12 and earlier to estimate \code{k}.

The \code{method} argument specifies one of three options to speed up the original dynamic programming taking a runtime cubic in sample size \var{n}. The default \code{"linear"} option, giving a total runtime of \eqn{O(n \lg n + kn)}{O(n lg n + kn)} or \eqn{O(kn)}{O(kn)} (if \code{x} is already sorted in ascending order) is the fastest option but uses the most memory (still \eqn{O(kn)}{O(kn)}) \insertCite{song2020wuc}{Ckmeans.1d.dp}; the \code{"loglinear"} option, with a runtime of \eqn{O(kn \lg n)}{O(kn lg n)}, is slightly slower but uses the least memory \insertCite{song2020wuc}{Ckmeans.1d.dp}; the slowest \code{"quadratic"} option \insertCite{Wang2011Ckmeans}{Ckmeans.1d.dp}, with a runtime of \eqn{O(kn^2)}{O(kn^2)}, is provided for the purpose of testing on small data sets.

When the sample size \var{n} is too large to create two \eqn{k \times n}{k x n} dynamic programming matrices in memory, we recommend the heuristic solutions implemented in the \code{\link{kmeans}} function in package \pkg{\link{stats}}.
}

\value{
	An object of class "\code{Ckmeans.1d.dp}" or "\code{Ckmedian.1d.dp}". It is a list containing the following components:

	\item{cluster}{a vector of clusters assigned to each element in \code{x}. Each cluster is indexed by an integer from 1 to \code{k}.}

	\item{centers}{a numeric vector of the (weighted) means for each cluster.}

	\item{withinss}{a numeric vector of the (weighted) within-cluster sum of squares for each cluster.}

	\item{size}{a vector of the (weighted) number of elements in each cluster.}

	\item{totss}{total sum of (weighted) squared distances between each element and the sample mean. This statistic is not dependent on the clustering result.}

	\item{tot.withinss}{total sum of (weighted) within-cluster squared distances between each element and its cluster mean. This statistic is minimized given the number of clusters.}

	\item{betweenss}{sum of (weighted) squared distances between each cluster mean and sample mean. This statistic is maximized given the number of clusters.}

	\item{xname}{a character string. The actual name of the \code{x} argument.}

	\item{yname}{a character string. The actual name of the \code{y} argument.}

Each class has a print and a plot method, which are described along with \code{\link{print.Ckmeans.1d.dp}} and \code{\link{plot.Ckmeans.1d.dp}}.
}

\references{
\insertAllCited{}
}

\seealso{
\code{\link{ahist}}, \code{\link{plot.Ckmeans.1d.dp}}, \code{\link{print.Ckmeans.1d.dp}} in this package.

\code{\link[stats]{kmeans}} in package \pkg{\link{stats}} that implements several heuristic \eqn{k}{k}-means algorithms.
}

\examples{
# Ex. 1 The number of clusters is provided.
# Generate data from a Gaussian mixture model of three components
x <- c(rnorm(50, sd=0.2), rnorm(50, mean=1, sd=0.3), rnorm(100,
       mean=-1, sd=0.25))
# Divide x into 3 clusters
k <- 3

result <- Ckmedian.1d.dp(x, k)

plot(result, main="Optimal univariate k-median given k")

result <- Ckmeans.1d.dp(x, k)

plot(result, main="Optimal univariate k-means given k")

plot(x, col=result$cluster, pch=result$cluster, cex=1.5,
     main="Optimal univariate k-means clustering given k",
     sub=paste("Number of clusters given:", k))
abline(h=result$centers, col=1:k, lty="dashed", lwd=2)
legend("bottomleft", paste("Cluster", 1:k), col=1:k, pch=1:k,
       cex=1.5, bty="n")

# Ex. 2 The number of clusters is determined by Bayesian
#       information criterion
# Generate data from a Gaussian mixture model of three components
x <- c(rnorm(50, mean=-3, sd=1), rnorm(50, mean=0, sd=.5),
       rnorm(50, mean=3, sd=1))
# Divide x into k clusters, k automatically selected (default: 1~9)

result <- Ckmedian.1d.dp(x)

plot(result, main="Optimal univariate k-median with k estimated")

result <- Ckmeans.1d.dp(x)

plot(result, main="Optimal univariate k-means with k estimated")

k <- max(result$cluster)
plot(x, col=result$cluster, pch=result$cluster, cex=1.5,
     main="Optimal univariate k-means clustering with k estimated",
     sub=paste("Number of clusters is estimated to be", k))
abline(h=result$centers, col=1:k, lty="dashed", lwd=2)
legend("topleft", paste("Cluster", 1:k), col=1:k, pch=1:k,
       cex=1.5, bty="n")

# Ex. 3 Segmenting a time course using optimal weighted
#       univariate clustering
n <- 160
t <- seq(0, 2*pi*2, length=n)
n1 <- 1:(n/2)
n2 <- (max(n1)+1):n
y1 <- abs(sin(1.5*t[n1]) + 0.1*rnorm(length(n1)))
y2 <- abs(sin(0.5*t[n2]) + 0.1*rnorm(length(n2)))
y <- c(y1, y2)

w <- y^8 # stress the peaks
res <- Ckmeans.1d.dp(t, k=c(1:10), w)
plot(res)
plot(t, w, main = "Time course weighted k-means",
     col=res$cluster, pch=res$cluster,
     xlab="Time t", ylab="Transformed intensity w",
     type="h")
abline(v=res$centers, col="chocolate", lty="dashed")
text(res$centers, max(w) * .95, cex=0.5, font=2,
     paste(round(res$size / sum(res$size) * 100), "/ 100"))
}


\keyword{ univar }
\keyword{ cluster }

\name{Univariate Clustering}
\alias{Univariate Clustering}
\alias{Ckmeans.1d.dp}
\alias{Ckmedian.1d.dp}

\title{Optimal (Weighted) Univariate Clustering}

\description{
Perform optimal univariate \eqn{k}{k}-means or \eqn{k}{k}-median clustering in linear (fastest), loglinear, or quadratic (slowest) time.
}

\usage{
Ckmeans.1d.dp(x, k=c(1,9), y=1,
              method=c("linear", "loglinear", "quadratic"),
              estimate.k=c("BIC", "BIC 3.4.12"))

Ckmedian.1d.dp(x, k=c(1,9), y=1,
               method=c("linear", "loglinear", "quadratic"),
               estimate.k=c("BIC", "BIC 3.4.12"))
}

\arguments{
  \item{x}{a numeric vector of data to be clustered. All \code{NA} elements must be removed from \code{x} before calling this function. The function will run faster on sorted \code{x} (in non-decreasing order) than an unsorted input.}

  \item{k}{either an exact integer number of clusters, or a vector of length two specifying the minimum and maximum numbers of clusters to be examined. The default is \code{c(1,9)}. When \code{k} is a range, the actual number of clusters is determined by Bayesian information criterion.}

  \item{y}{a value of 1 (default) to specify equal weights of 1 for each element in \code{x}, or a numeric vector of unequal non-negative weights for each element in \code{x}. It is highly recommended to use positive (instead of zero) weights to account for the influence of every element. The weights have a strong impact on the clustering result. When the number of clusters \code{k} is given as a range, the weights should be linearly scaled to sum up to the observed sample size. Currently, \code{Ckmedian.1d.dp} only works with an equal weight of 1.}

  \item{method}{a character string to specify the speedup method to the original cubic runtime dynamic programming. The default is \code{"linear"}. All methods generate the same optimal results but differ in runtime or memory usage. See Details.}

  \item{estimate.k}{a character string to specify the method to estimate optimal \code{k}. This argument is effective only when a range for \code{k} is provided. The default is \code{"BIC"}. See Details.}

}

\author{
	Joe Song and Haizhou Wang
}

\details{
\code{Ckmean.1d.dp} minimizes unweighted or weighted within-cluster sum of squared distance (L2).

\code{Ckmedian.1d.dp} minimizes within-cluster sum of distance (L1). Only unweighted solution is implemented and guarantees optimality.

In contrast to the heuristic \var{k}-means algorithms implemented in function \code{\link{kmeans}}, this function optimally assigns elements in numeric vector \code{x} into \code{k} clusters by dynamic programming \insertCite{Wang2011Ckmeans,song2020wuc}{Ckmeans.1d.dp}. It minimizes the total of within-cluster sums of squared distances (\var{withinss}) between each element and its corresponding cluster mean. When a range is provided for \code{k}, the exact number of clusters is determined by Bayesian information criterion \insertCite{song2020wuc}{Ckmeans.1d.dp}. Different from the heuristic \var{k}-means algorithms whose results may be non-optimal or change from run to run, the result of Ckmeans.1d.dp is guaranteed to be optimal and reproducible, and its advantage in efficiency and accuracy over heuristic \var{k}-means methods is most pronounced at large \var{k}.

The \code{estimate.k} argument specifies the method to select optimal \code{k} based on the Gaussian mixture model using the Bayesian information criterion (BIC). When \code{estimate.k="BIC"}, it effectively deals with variance estimation for a cluster with identical values. When \code{estimate.k="BIC 3.4.12"}, it uses the code in version 3.4.12 and earlier to estimate \code{k}.

The \code{method} argument specifies one of three options to speed up the original dynamic programming taking a runtime cubic in sample size \var{n}. The default \code{"linear"} option, giving a total runtime of \eqn{O(n \lg n + kn)}{O(n lg n + kn)} or \eqn{O(kn)}{O(kn)} (if \code{x} is already sorted in ascending order) is the fastest option but uses the most memory (still \eqn{O(kn)}{O(kn)}) \insertCite{song2020wuc}{Ckmeans.1d.dp}; the \code{"loglinear"} option, with a runtime of \eqn{O(kn \lg n)}{O(kn lg n)}, is slightly slower but uses the least memory \insertCite{song2020wuc}{Ckmeans.1d.dp}; the slowest \code{"quadratic"} option \insertCite{Wang2011Ckmeans}{Ckmeans.1d.dp}, with a runtime of \eqn{O(kn^2)}{O(kn^2)}, is provided for the purpose of testing on small data sets.

When the sample size \var{n} is too large to create two \eqn{k \times n}{k x n} dynamic programming matrices in memory, we recommend the heuristic solutions implemented in the \code{\link{kmeans}} function in package \pkg{\link{stats}}.
}

\value{
	An object of class "\code{Ckmeans.1d.dp}" or "\code{Ckmedian.1d.dp}". It is a list containing the following components:

	\item{cluster}{a vector of clusters assigned to each element in \code{x}. Each cluster is indexed by an integer from 1 to \code{k}.}

	\item{centers}{a numeric vector of the (weighted) means for each cluster.}

	\item{withinss}{a numeric vector of the (weighted) within-cluster sum of squares for each cluster.}

	\item{size}{a vector of the (weighted) number of elements in each cluster.}

	\item{totss}{total sum of (weighted) squared distances between each element and the sample mean. This statistic is not dependent on the clustering result.}

	\item{tot.withinss}{total sum of (weighted) within-cluster squared distances between each element and its cluster mean. This statistic is minimized given the number of clusters.}

	\item{betweenss}{sum of (weighted) squared distances between each cluster mean and sample mean. This statistic is maximized given the number of clusters.}

	\item{xname}{a character string. The actual name of the \code{x} argument.}

	\item{yname}{a character string. The actual name of the \code{y} argument.}

Each class has a print and a plot method, which are described along with \code{\link{print.Ckmeans.1d.dp}} and \code{\link{plot.Ckmeans.1d.dp}}.
}

\references{
\insertAllCited{}
}

\seealso{
\code{\link{ahist}}, \code{\link{plot.Ckmeans.1d.dp}}, \code{\link{print.Ckmeans.1d.dp}} in this package.

\code{\link[stats]{kmeans}} in package \pkg{\link{stats}} that implements several heuristic \eqn{k}{k}-means algorithms.
}

\examples{
# Ex. 1 The number of clusters is provided.
# Generate data from a Gaussian mixture model of three components
x <- c(rnorm(50, sd=0.2), rnorm(50, mean=1, sd=0.3), rnorm(100,
       mean=-1, sd=0.25))
# Divide x into 3 clusters
k <- 3

result <- Ckmedian.1d.dp(x, k)

plot(result, main="Optimal univariate k-median given k")

result <- Ckmeans.1d.dp(x, k)

plot(result, main="Optimal univariate k-means given k")

plot(x, col=result$cluster, pch=result$cluster, cex=1.5,
     main="Optimal univariate k-means clustering given k",
     sub=paste("Number of clusters given:", k))
abline(h=result$centers, col=1:k, lty="dashed", lwd=2)
legend("bottomleft", paste("Cluster", 1:k), col=1:k, pch=1:k,
       cex=1.5, bty="n")

# Ex. 2 The number of clusters is determined by Bayesian
#       information criterion
# Generate data from a Gaussian mixture model of three components
x <- c(rnorm(50, mean=-3, sd=1), rnorm(50, mean=0, sd=.5),
       rnorm(50, mean=3, sd=1))
# Divide x into k clusters, k automatically selected (default: 1~9)

result <- Ckmedian.1d.dp(x)

plot(result, main="Optimal univariate k-median with k estimated")

result <- Ckmeans.1d.dp(x)

plot(result, main="Optimal univariate k-means with k estimated")

k <- max(result$cluster)
plot(x, col=result$cluster, pch=result$cluster, cex=1.5,
     main="Optimal univariate k-means clustering with k estimated",
     sub=paste("Number of clusters is estimated to be", k))
abline(h=result$centers, col=1:k, lty="dashed", lwd=2)
legend("topleft", paste("Cluster", 1:k), col=1:k, pch=1:k,
       cex=1.5, bty="n")

# Ex. 3 Segmenting a time course using optimal weighted
#       univariate clustering
n <- 160
t <- seq(0, 2*pi*2, length=n)
n1 <- 1:(n/2)
n2 <- (max(n1)+1):n
y1 <- abs(sin(1.5*t[n1]) + 0.1*rnorm(length(n1)))
y2 <- abs(sin(0.5*t[n2]) + 0.1*rnorm(length(n2)))
y <- c(y1, y2)

w <- y^8 # stress the peaks
res <- Ckmeans.1d.dp(t, k=c(1:10), w)
plot(res)
plot(t, w, main = "Time course weighted k-means",
     col=res$cluster, pch=res$cluster,
     xlab="Time t", ylab="Transformed intensity w",
     type="h")
abline(v=res$centers, col="chocolate", lty="dashed")
text(res$centers, max(w) * .95, cex=0.5, font=2,
     paste(round(res$size / sum(res$size) * 100), "/ 100"))
}


\keyword{ univar }
\keyword{ cluster }

\name{MultiChannel.WUC}
\alias{MultiChannel.WUC}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
  Optimal Multi-channel Weighted Univariate Clustering
}
\description{
  Perform optimal multi-channel weighted univariate \eqn{k}{k}-means clustering in linear time.
}
\usage{
  MultiChannel.WUC(x, y,  k=c(1,9))
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{a numeric vector of data to be clustered. All \code{NA} elements must be removed from \code{x} before calling this function. The function will run faster on sorted \code{x} (in non-decreasing order) than an unsorted input.}

  \item{y}{a numeric matrix of non-negative weights for each element in \code{x}. Columns of the matrix are channels. It is highly recommended to use positive (instead of zero) weights to account for the influence of every element. Weights strongly influence clustering results. When the number of clusters \code{k} is given as a range, the weights should be linearly scaled to sum up to the observed sample size.}

  \item{k}{either an exact integer number of clusters, or a vector of length two specifying the minimum and maximum numbers of clusters to be examined. The default is \code{c(1,9)}. When \code{k} is a range, the actual number of clusters is determined by Bayesian information criterion (BIC).}
}
\details{
  \code{MultiChannel.WUC} minimizes the total weighted within-cluster sum of squared distance \insertCite{zhong2019modelfree}{Ckmeans.1d.dp}. It uses the SMAWK algorithm \insertCite{aggarwal1987geometric}{Ckmeans.1d.dp} with modified data structure to speed up the dynamic programming to linear runtime. The method selects an optimal \code{k} based on an approximate Gaussian mixture model using the BIC.
}
\value{
  A list object containing the following components:

	\item{cluster}{a vector of clusters assigned to each element in \code{x}. Each cluster is indexed by an integer from 1 to \code{k}.}

	\item{centers}{a numeric vector of the (weighted) means for each cluster.}

	\item{withinss}{a numeric vector of the (weighted) within-cluster sum of squares for each cluster.}

	\item{size}{a vector of the (weighted) number of elements in each cluster.}

	\item{totss}{total sum of (weighted) squared distances between each element and the sample mean. This statistic is not dependent on the clustering result.}

	\item{tot.withinss}{total sum of (weighted) within-cluster squared distances between each element and its cluster mean. This statistic is minimized given the number of clusters.}

	\item{betweenss}{sum of (weighted) squared distances between each cluster mean and sample mean. This statistic is maximized given the number of clusters.}

	\item{xname}{a character string. The actual name of the \code{x} argument.}

	\item{yname}{a character string. The actual name of the \code{y} argument.}
}
\references{
  \insertAllCited{}
}
\author{
  Hua Zhong and Mingzhou Song
}
\examples{
  x <- sample(x = c(1:100), size = 20, replace = TRUE)
  Y <- matrix(sample(x = c(1:100), size = 40, replace = TRUE), ncol=2, nrow=length(x))

  res <- MultiChannel.WUC(x = x, y = Y, k = c(1:10))
  plot(res)

  n <- c(20, 20, 20)
  x <- c(rnorm(n[1], mean=-6),
         rnorm(n[2], mean=0),
         rnorm(n[3], mean=6))

  Y <- matrix(c(
    rep(c(1,0,0), times=n[1]),
    rep(c(0,1,0), times=n[2]),
    rep(c(0,0,1), times=n[3])
  ), byrow=TRUE, nrow=length(x))

  res <- MultiChannel.WUC(x = x, y = Y, k = 3)

  opar <- par(mar=c(3,3,2.5,1), mgp=c(1.5,0.5,0))
  plot(res)
  par(opar)

}

\keyword{ cluster }
\keyword{ distribution }

\name{plotBIC}
\alias{plotBIC}
\title{Plot Bayesian Information Criterion as a Function of Number of Clusters}
\description{
Plot Bayesian information criterion (BIC) as a function of the number of clusters obtained from optimal univariate clustering results returned from \code{Ckmeans.1d.dp}. The BIC normalized by sample size (BIC/n) is shown.
}
\usage{
plotBIC(
  ck, xlab="Number of clusters k",
  ylab = "BIC/n", type="b",
  sub=paste("n =", length(ck$cluster)),
  main=paste("Bayesian information criterion",
             "(normalized by sample size)", sep="\n"),
  ...
)
}

\arguments{
  \item{ck}{an object of class \code{Ckmeans.1d.dp} returned by \code{\link{Ckmeans.1d.dp}}.}

 \item{xlab}{a character string. The x-axis label for the plot.}

 \item{ylab}{a character string. The x-axis label for the plot.}

 \item{type}{the type of plot to be drawn. See \code{\link[graphics]{plot}}.}

 \item{main}{a character string. The title for the plot.}

 \item{sub}{a character string. The subtitle for the plot.}

 \item{...}{arguments passed to \code{\link{plot}} function in package \pkg{graphics}.}
}

\author{
	Joe Song
}

\details{
The function visualizes the input data as sticks whose heights are the weights. It uses different colors to indicate optimal \var{k}-means clusters. The method to calcualte BIC based on Gaussian mixture models estimated on a univariate clustering is described in \insertCite{song2020wuc}{Ckmeans.1d.dp}.
}

\value{
	An object of class "\code{Ckmeans.1d.dp}" defined in \code{\link{Ckmeans.1d.dp}}.
}

\examples{
# Example: clustering data generated from a Gaussian mixture
#          model of two components
x <- rnorm(50, mean=-1, sd=0.3)
x <- append(x, rnorm(50, mean=1, sd=0.3) )
res <- Ckmeans.1d.dp(x)
plotBIC(res)

y <- (rnorm(length(x)))^2
res <- Ckmeans.1d.dp(x, y=y)
plotBIC(res)
}

\references{
\insertAllCited{}
}

\keyword{ univar }
\keyword{ cluster }
\keyword{ hplot }

\name{ahist}
\alias{ahist}
\title{Adaptive Histograms}
\description{
Generate or plot histograms adaptive to patterns in univariate data. The number and widths of histogram bins are automatically calculated based on an optimal univariate clustering of input data. Thus the bins are unlikely of equal width.
}

\usage{
ahist(x, k = c(1,9), breaks=NULL, data=NULL, weight=1,
      plot = TRUE, xlab = deparse(substitute(x)),
      wlab = deparse(substitute(weight)),
      main = NULL, col = "lavender", border = graphics::par("fg"),
      lwd = graphics::par("lwd"),
      col.stick = "gray", lwd.stick = 1, add.sticks=TRUE,
      style = c("discontinuous", "midpoints"),
      skip.empty.bin.color=TRUE,
      \dots)
}

\arguments{
  \item{x}{a numeric vector of data or an object of class \code{"Ckmeans.1d.dp"}.

If \code{x} is a numeric vector, all \code{NA} elements must be removed from \code{x} before calling this function.

If \code{x} is an object of class \code{"Ckmeans.1d.dp"}, the clustering information in \code{x} will be used and the \code{data} argument contains the numeric vector to be plotted.}

  \item{k}{either an exact integer number of bins/clusters, or a vector of length two specifying the minimum and maximum numbers of bins/clusters to be examined. The default is \code{c(1,9)}. When \code{k} is a range, the actual number of clusters is determined by Bayesian information criterion. This argument is ignored if \code{x} is an object of class \code{"Ckmeans.1d.dp"}.}

  \item{breaks}{This argument is defined in \code{\link{hist}}. If this argument is provided, optimal univariate clustering is not applied to obtain the histogram, but instead the histogram will be generated by the \code{\link{hist}} function in \pkg{\link{graphics}}, except that sticks representing data can still be optionally plotted by specifying the \code{add.sticks=TRUE} argument.}

  \item{data}{a numeric vector. If \code{x} is an object of class \code{"Ckmeans.1d.dp"}, the data argument must be provided. If \code{x} is a numeric vector, this argument is ignored.}

  \item{weight}{a value of 1 to specify equal weights or a numeric vector of unequal weights for each element. The default weight is one. It is highly recommended to use positive (instead of zero) weights to account for the influence of every element. The weights have a strong impact on the clustering result.}

 \item{plot}{a logical. If \code{TRUE}, the histogram will be plotted.}

 \item{xlab}{a character string. The x-axis label for the plot.}

 \item{wlab}{a character string. The weight-axis label for the plot. It is the vertical axis to the right of the plot.}

 \item{main}{a character string. The title for the plot.}

 \item{col}{a character string. The fill color of the histogram bars.}

 \item{border}{a character string. The color of the histogram bar borders.}

 \item{lwd}{a numeric value. The line width of the border of the histogram bars}

 \item{col.stick}{a character string. The color of the sticks above the x-axis. See Details.}

 \item{lwd.stick}{a numeric value. The line width of the sticks above the x-axis. See Details.}

 \item{add.sticks}{a logical. If \code{TRUE} (default), the sticks representing the data will be added to the bottom of the histogram. Otherwise, sticks are not plotted.}

 \item{style}{a character string. The style of the adaptive histogram. See details.}

 \item{skip.empty.bin.color}{a logical. If \code{TRUE} (default), an empty bin (invisible) will be assigned the same bar color with the next bin. This is useful when all provided colors are to be used for non-empty bins. If \code{FALSE}, each bin will be assigned a bar color from \code{col}. A value of \code{TRUE} will coordinate the bar and stick colors.}

 \item{...}{additional arguments to be passed to \code{\link{hist}} or \code{\link{plot.histogram}}.}
}

\author{
	Joe Song
}

\details{
The histogram is by default plotted using the \code{\link{plot.histogram}} method. The plot can be optionally disabled with the \code{plot=FALSE} argument. The original input data are shown as sticks just above the horizontal axis.

If the \code{breaks} argument is not specified, the number of histogram bins is the optimal number of clusters estimated using Bayesian information criterion evaluated on Gaussian mixture models fitted to the input data in \code{x}.

If not provided with the \code{breaks} argument, breaks in the histogram are derived from clusters identified by optimal univariate clustering (\code{\link{Ckmeans.1d.dp}}) in two styles. With the default \code{"discontinuous"} style,  the bin width of each bar is determined according to a data-adaptive rule; the \code{"midpoints"} style uses the midpoints of cluster border points to determine the bin-width. For clustered data, the \code{"midpoints"} style generates bins that are too wide to capture the cluster patterns. In contrast, the \code{"discontinuous"} style is more adaptive to the data by allowing some bins to be empty making the histogram bars discontinuous.}

\value{
An object of class \code{histogram} defined in \code{\link{hist}}. It has a S3 \code{plot} method \code{\link{plot.histogram}}.
}

\seealso{
  \code{\link{hist}} in package \pkg{\link{graphics}}.
}

\examples{
# Example 1: plot an adaptive histogram from data generated by
#   a Gaussian mixture model with three components
x <- c(rnorm(40, mean=-2, sd=0.3),
       rnorm(45, mean=1, sd=0.1),
       rnorm(70, mean=3, sd=0.2))
ahist(x, col="lightblue", sub=paste("n =", length(x)),
      col.stick="salmon", lwd=2,
      main=paste("Example 1. Gaussian mixture model with 3 components",
                 "(one bin per component)", sep="\n"))


# Example 2: plot an adaptive histogram from data generated by
#   a Gaussian mixture model with three components using a given
#   number of bins
ahist(x, k=9, col="lavender", col.stick="salmon",
      sub=paste("n =", length(x)), lwd=2,
      main=paste("Example 2. Gaussian mixture model with 3 components",
                 "(on average 3 bins per component)", sep="\n"))

# Example 3: The DNase data frame has 176 rows and 3 columns of
#   data obtained during development of an ELISA assay for the
#   recombinant protein DNase in rat serum.

data(DNase)
res <- Ckmeans.1d.dp(DNase$density)
kopt <- length(res$size)
ahist(res, data=DNase$density, col=rainbow(kopt),
      col.stick=rainbow(kopt)[res$cluster],
      sub=paste("n =", length(x)), border="transparent",
      xlab="Optical density of protein DNase",
      main="Example 3. Elisa assay of DNase in rat serum")


# Example 4: Add sticks to histograms with the R provided
#   hist() function.

ahist(DNase$density, breaks="Sturges", col="palegreen",
      add.sticks=TRUE, col.stick="darkgreen",
      main=paste("Example 4. Elisa assay of DNase in rat serum",
                 "(Equal width bins)", sep="\n"),
      xlab="Optical density of protein DNase")

# Example 5: Weighted adatpive histograms

x <- sort(c(rnorm(40, mean=-2, sd=0.3),
       rnorm(45, mean=2, sd=0.1),
       rnorm(70, mean=4, sd=0.2)))

y <- (1 + sin(0.10 * seq_along(x))) * (x-1)^2

ahist(x, weight=y, sub=paste("n =", length(x)),
      col.stick="forestgreen", lwd.stick=0.25, lwd=2,
      main="Example 5. Weighted adaptive histogram")


# Example 6: Cluster data with repetitive elements

ahist(c(1,1,1,1, 3,4,4, 6,6,6), k=c(2,4), col="gray",
      lwd=2, lwd.stick=6, col.stick="chocolate",
      main=paste("Example 6. Adaptive histogram of",
                 "repetitive elements", sep="\n"))

}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }% use one of  RShowDoc("KEYWORDS")
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\keyword{ univar }
\keyword{ cluster }
\keyword{ hplot }
\keyword{ distribution }

\name{print.Cksegs.1d.dp}
\alias{print.Cksegs.1d.dp}

\title{Print Optimal Univariate Segmentation Results}
\description{
Print optimal univariate segmentation results obtained from \code{Cksegs.1d.dp}.
}
\usage{
\method{print}{Cksegs.1d.dp}(x, \dots)
}
\arguments{
  \item{x}{object returned by calling \code{Cksegs.1d.dp}.}
  \item{...}{arguments passed to \code{\link{print}} function.}
}

\author{
	Joe Song
}

\details{
Function \code{print.Cksegs.1d.dp} prints the maximum ratio of between-cluster sum of squares to total sum of squares unless all input elements are zero. The ratio is an indicator of maximum achievable clustering quality given the number of clusters: 100\% for a perfect clustering and 0\% for no cluster patterns.
}

\value{
	An object of class "\code{Cksegs.1d.dp}" as defined in \code{\link{Cksegs.1d.dp}}.
}

\examples{
# Example: clustering data generated from a Gaussian
#          mixture model of two components
x <- c(rnorm(15, mean=-1, sd=0.3),
       rnorm(15, mean=1, sd=0.3))

y <- x^3
res <- Cksegs.1d.dp(y, x=x)
print(res, lwd=2)
}

\keyword{ univar }
\keyword{ cluster }
\keyword{ print }

\name{Ckmeans.1d.dp-package}
\alias{Ckmeans.1d.dp-package}
\docType{package}
\title{
Optimal, Fast, and Reproducible Univariate Clustering
}
\description{
This package provides a powerful set of tools for univariate data analysis with guaranteed optimality, efficiency, and reproducibility.
Four problems including univariate \eqn{k}{k}-means, \eqn{k}{k}-median, \eqn{k}{k}-segments, and multi-channel weighted \eqn{k}{k}-means are solved with guaranteed optimality and reproducibility. The core algorithm minimizes the (weighted) sum of within-cluster distances using respective metrics. Its advantage over heuristic clustering in efficiency and accuracy is increasingly pronounced as the number of clusters \eqn{k}{k} increases. Weighted \eqn{k}{k}-means can also optimally segment time series to perform peak calling. An auxiliary function generates histograms that are adaptive to patterns in data.

The Ckmeans.1d.dp algorithm clusters (weighted) univariate data given by a numeric vector \eqn{x}{x} into \eqn{k}{k} groups by dynamic programming \insertCite{Wang2011Ckmeans,song2020wuc}{Ckmeans.1d.dp}. It guarantees the optimality of clustering---the total of within-cluster sums of squares is always the minimum given the number of clusters \eqn{k}{k}. In contrast, heuristic univariate clustering algorithms may be non-optimal or inconsistent from run to run. As non-negative weights are supported, the algorithm can partition a time course using time points as input and corresponding values as weight. Based on an optimal clustering, a function can generate histograms adaptive to patterns in data. The implementation of this algorithm is numerically stable.

A linear time solution. Excluding the time for sorting \eqn{x}{x}, the weighted univariate clustering algorithm takes a runtime of \eqn{O(kn)}{O(kn)} \insertCite{song2020wuc}{Ckmeans.1d.dp}, linear in both sample size \eqn{n}{n} and the number of clusters \eqn{k}{k}. The approach was first introduced into version 3.4.9 (not publicly released) on July 16, 2016, using a new divide-and-conquer strategy integrating a previous theoretical result on matrix search \insertCite{aggarwal1987geometric}{Ckmeans.1d.dp} and a novel in-place search space reduction method \insertCite{song2020wuc}{Ckmeans.1d.dp}.

A log-linear time solution. Since version 3.4.6, a divide-and-conquer strategy that is simple to code reduces the runtime from \eqn{O(kn^2)}{O(kn^2)} down to \eqn{O(kn\lg n)}{O(kn lg n)} \insertCite{song2020wuc}{Ckmeans.1d.dp}.

A quadratic time solution. Before version 3.4.6, Ckmeans.1d.dp uses an algorithm that runs in quadratic runtime \eqn{O(kn^2)}{O(kn^2)} \insertCite{Wang2011Ckmeans}{Ckmeans.1d.dp}.

A cubic time solution. \insertCite{Bellman1973;textual}{Ckmeans.1d.dp} first described a general dynamic programming strategy for solving univariate clustering problems with additive optimality measures. The strategy, however, did not address any specific characteristics of the \eqn{k}{k}-means problem and its implied general algorithm will have a time complexity of \eqn{O(kn^3)}{O(kn^3)} on an input of \eqn{n}{n} points.

The space complexity in all cases is \eqn{O(kn)}{O(kn)}.

This package provides a powerful alternative to heuristic clustering and also new functionality for weighted clustering, segmentation, and peak calling with guaranteed optimality. It is practical for Ckmeans.1d.dp to find a few clusters on millions of sample points with optional weights in seconds using a single core on a typical desktop computer.

Third parties have ported various versions of this package to JavaScript, MATLAB, Python, Ruby, SAS, and Scala.
}

\details{
The most important function of this package is \code{\link{Ckmeans.1d.dp}} that implements optimal univariate clustering either weighted or unweighted. It also contains an adaptive histogram function \code{\link{ahist}}, plotting functions \code{\link{plot.Ckmeans.1d.dp}} and \code{\link{plotBIC}}, and a print function \code{\link{print.Ckmeans.1d.dp}}.
}

\seealso{
  The \code{\link{kmeans}} function in package \pkg{\link{stats}} that implements several heuristic \eqn{k}{k}-means algorithms.
}

\section{Disclaimer}{
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see \url{https://www.gnu.org/licenses/}.}

\author{
Joe Song, Hua Zhong, and Haizhou Wang
}

\references{

  \insertAllCited{}

% Bellman, R. (1973) A note on cluster analysis and dynamic programming. \emph{Mathematical Biosciences} \bold{18}(3), 311--312.

% Aggarwal, A., Klawe, M. M., Moran, S., Shor, P., and Wilber, R. (1987) Geometric applications of a matrix-searching algorithm. \emph{Algorithmica} \bold{2}(1-4), 195--208.

% Wang, H. and Song, M. (2011) Ckmeans.1d.dp: optimal \var{k}-means clustering in one dimension by dynamic programming. \emph{The R Journal} \bold{3}(2), 29--33. Retrieved from \url{https://journal.r-project.org/archive/2011-2/RJournal_2011-2_Wang+Song.pdf}
}

\keyword{ package }
\keyword{ cluster }
\keyword{ univar }

\name{plot.Ckmeans.1d.dp}
\alias{plot.Ckmeans.1d.dp}
\alias{plot.Ckmedian.1d.dp}
\title{Plot Optimal Univariate Clustering Results}
\description{
Plot optimal univariate clustering results returned from \code{Ckmeans.1d.dp}.
}
\usage{
\method{plot}{Ckmeans.1d.dp}(x, xlab=NULL, ylab=NULL, main=NULL,
      sub=NULL, col.clusters=NULL, \dots)
\method{plot}{Ckmedian.1d.dp}(x, xlab=NULL, ylab=NULL, main=NULL,
      sub=NULL, col.clusters=NULL, \dots)
}
%(x, xlab=ck$xname,
%     ylab=ifelse(x$yname=="1", "Weight", x$yname),
%     main="Optimal univariate clustering of", x$xname),
%     sub=paste("n =", length(x$cluster)),
%     col.clusters=seq_along(x$size),
%     \dots)

\arguments{
  \item{x}{an object of class as returned by \code{\link{Ckmeans.1d.dp}} or \code{\link{Ckmedian.1d.dp}}.}

 \item{xlab}{a character string. The x-axis label for the plot.}

  \item{ylab}{a character string. The x-axis label for the plot.}

 \item{main}{a character string. The title for the plot.}

  \item{sub}{a character string. The subtitle for the plot.}

  \item{col.clusters}{a vector of colors, defined either by integers or by color names. If the length is shorter than the number of clusters, the colors will be reused.}

  \item{...}{arguments passed to \code{\link{plot}} function in package \pkg{graphics}.}
}

\author{
	Joe Song
}

\details{
The functions \code{plot.Ckmeans.1d.dp} and \code{plot.Ckmedian.1d.dp} visualize the input data as sticks whose heights are the weights. They use different colors to indicate clusters.
}

\value{
	An object of class "\code{Ckmeans.1d.dp}" or "\code{Ckmedian.1d.dp}" defined in \code{\link{Ckmeans.1d.dp}}.
}

\examples{
# Example: clustering data generated from a Gaussian
#          mixture model of three components
x <- c(rnorm(50, mean=-1, sd=0.3),
       rnorm(50, mean=1, sd=0.3),
       rnorm(50, mean=3, sd=0.3))

res <- Ckmeans.1d.dp(x)
plot(res)

y <- (rnorm(length(x)))^2
res <- Ckmeans.1d.dp(x, y=y)
plot(res)

res <- Ckmedian.1d.dp(x)
plot(res)
}

\keyword{ univar }
\keyword{ cluster }
\keyword{ hplot }
\keyword{ distribution }

\name{plot.MultiChannelClusters}
\alias{plot.MultiChannelClusters}
\title{Plot Multi-Channel Clustering Results}
\description{
Plot multi-channel clustering results returned from \code{MultiChannel.WUC}.
}
\usage{
\method{plot}{MultiChannelClusters}(x, xlab=NULL, ylab=NULL, main=NULL,
      sub=NULL, col.clusters=NULL, \dots)
}

\arguments{
  \item{x}{an object of class as returned by \code{\link{MultiChannel.WUC}}.}

 \item{xlab}{a character string. The x-axis label for the plot.}

  \item{ylab}{a character string. The x-axis label for the plot.}

 \item{main}{a character string. The title for the plot.}

  \item{sub}{a character string. The subtitle for the plot.}

  \item{col.clusters}{a vector of colors, defined either by integers or by color names. If the length is shorter than the number of clusters, the colors will be reused.}

  \item{...}{arguments passed to \code{\link{plot}} function in package \pkg{graphics}.}
}

\author{
	Joe Song
}

\details{
The function visualizes the input data as sticks whose heights are the weights. Colors indicate clusters. See \code{\link{MultiChannel.WUC}} for examples.
}

\value{
	An object of class "\code{MultiChannelClusters}" defined in \code{\link{MultiChannel.WUC}}.
}

\keyword{ cluster }
\keyword{ hplot }
\keyword{ distribution }

\name{Univariate Segmentation}
\alias{Univariate Segmentation}
\alias{Cksegs.1d.dp}

\title{Optimal Univariate Segmentation}

\description{
Perform optimal univariate \eqn{k}{k}-segmentation.
}

\usage{
Cksegs.1d.dp(y, k=c(1,9), x=seq_along(y),
             method=c("quadratic", "linear", "loglinear"),
             estimate.k=c("BIC", "BIC 3.4.12"))
}

\arguments{

  \item{y}{a numeric vector of y values. Values can be negative.}

  \item{k}{either an exact integer number of clusters, or a vector of length two specifying the minimum and maximum numbers of clusters to be examined. The default is \code{c(1,9)}. When \code{k} is a range, the actual number of clusters is determined by Bayesian information criterion.}

  \item{x}{an optional numeric vector of data to be clustered. All \code{NA} elements must be removed from \code{x} before calling this function. The function will run faster on sorted \code{x} (in non-decreasing order) than an unsorted input.}

  \item{method}{a character string to specify the speedup method to the original cubic runtime dynamic programming. The default is \code{"quadratic"}, which generates optimal results. The other options do not guarantee optimal solution and differ in runtime or memory usage. See Details.}

  \item{estimate.k}{a character string to specify the method to estimate optimal \code{k}. The default is \code{"BIC"}. See Details.}

}

\author{
	Joe Song
}

\details{

\code{Cksegs.1d.dp} minimizes within-cluster sum of squared distance on \code{y}. It offers optimal piece-wise constant approximation of \code{y} within clusters of \code{x}. Only \code{method="quadratic"} guarantees optimality. The "linear" and "loglinear" options are faster but not always optimal and are provided for comparison purposes.

The Bayesian information criterion (BIC) method to select optimal \code{k} is updated to deal with duplicates in the data. Otherwise, the estimated k would be the same with previous versions. Set \code{estimate.k="BIC"} to use the latest method; use \code{estimate.k="BIC 3.4.12"} to use the BIC method in version 3.4.12 or earlier to estimated \code{k} from the given range. This option is effective only when a range for \code{k} is provided.

\code{method} specifies one of three options to speed up the original dynamic programming taking a runtime cubic in sample size \var{n}. The default \code{"quadratic"} option, with a runtime of \eqn{O(kn^2)}{O(kn^2)}, guarantees optimality. The next two options do not guarantee optimality. The \code{"linear"} option, giving a total runtime of \eqn{O(n \lg n + kn)}{O(n lg n + kn)} or \eqn{O(kn)}{O(kn)} (if \code{x} is already sorted in ascending order) is the fastest option but uses the most memory (still \eqn{O(kn)}{O(kn)}); the \code{"loglinear"} option, with a runtime of \eqn{O(kn \lg n)}{O(kn lg n)}, is slightly slower but uses the least memory.
}

\value{
	An object of class "\code{Cksegs.1d.dp}". It is a list containing the following components:

	\item{cluster}{a vector of clusters assigned to each element in \code{x}. Each cluster is indexed by an integer from 1 to \code{k}.}

	\item{centers}{a numeric vector of the (weighted) means for each cluster.}

	\item{withinss}{a numeric vector of the (weighted) within-cluster sum of squares for each cluster.}

	\item{size}{a vector of the (weighted) number of elements in each cluster.}

	\item{totss}{total sum of (weighted) squared distances between each element and the sample mean. This statistic is not dependent on the clustering result.}

	\item{tot.withinss}{total sum of (weighted) within-cluster squared distances between each element and its cluster mean. This statistic is minimized given the number of clusters.}

	\item{betweenss}{sum of (weighted) squared distances between each cluster mean and sample mean. This statistic is maximized given the number of clusters.}

	\item{xname}{a character string. The actual name of the \code{x} argument.}

	\item{yname}{a character string. The actual name of the \code{y} argument.}

The class has a print and a plot method: \code{\link{print.Cksegs.1d.dp}} and \code{\link{plot.Cksegs.1d.dp}}.
}

\seealso{
\code{\link{plot.Cksegs.1d.dp}} and \code{\link{print.Cksegs.1d.dp}}.
}

\examples{
# Ex 1. Segmenting by y

y <- c(1,1,1,2,2,2,4,4,4,4)

res <- Cksegs.1d.dp(y, k=c(1:10))

main <- "k-segs giving 3 clusters\nsucceeded in finding segments"

opar <- par(mfrow=c(1,2))

plot(res, main=main, xlab="x")

res <- Ckmeans.1d.dp(x=seq_along(y), k=c(1:10), y)
main <- "Weighted k-means giving 1 cluster\nfailed to find segments"

plot(res, main=main, xlab="x")

par(opar)

# Ex 2. Segmenting by y

y <- c(1,1,1.1,1, 2,2.5,2, 4,5,4,4)
res <- Cksegs.1d.dp(y, k=c(1:10))
plot(res, xlab="x")

# Ex 3. Segmenting a sinusoidal curve by y
x <- 1:125
y <- sin(x * .2)
res.q <- Cksegs.1d.dp(y, k=8, x=x)
plot(res.q, lwd=3, xlab="x")

# Ex 4. Segmenting by y

y <- rep(c(1,-3,4,-2), each=20)
y <- y + 0.5*rnorm(length(y))
k <- 1:10
res.q <- Cksegs.1d.dp(y, k=k, method="quadratic")
main <- paste("Cksegs (method=\"quadratic\"):\ntot.withinss =",
              format(res.q$tot.withinss, digits=4), "BIC =",
              format(res.q$BIC[length(res.q$size)], digits=4),
              "\nGUARANTEE TO BE OPTIMAL")
plot(res.q, main=main, xlab="x")
res.l <- Cksegs.1d.dp(y, k=k, method="linear")
main <- paste("Cksegs (method=\"linear\"):\ntot.withinss =",
               format(res.l$tot.withinss, digits=4), "BIC =",
              format(res.l$BIC[length(res.l$size)], digits=4),
               "\nFAST BUT MAY NOT BE OPTIMAL")
plot(res.l, main=main, xlab="x")
res.g <- Cksegs.1d.dp(y, k=k, method="loglinear")
main <- paste("Cksegs (method=\"loglinear\"):\ntot.withinss =",
              format(res.g$tot.withinss, digits=4), "BIC =",
              format(res.g$BIC[length(res.g$size)], digits=4),
              "\nFAST BUT MAY NOT BE OPTIMAL")
plot(res.g, main=main, xlab="x")
}


\keyword{ univar }
\keyword{ cluster }

\name{print.Ckmeans.1d.dp}
\alias{print.Ckmeans.1d.dp}
\alias{print.Ckmedian.1d.dp}

\title{Print Optimal Univariate Clustering Results}
\description{
Print optimal univariate clustering results obtained from \code{Ckmeans.1d.dp} or \code{Ckmedian.1d.dp}.
}

\usage{
\method{print}{Ckmeans.1d.dp}(x, \dots)
\method{print}{Ckmedian.1d.dp}(x, \dots)
}

\arguments{
  \item{x}{object returned by calling \code{Ckmeans.1d.dp} or \code{Cksegs.1d.dp}.}
  \item{...}{arguments passed to \code{\link{print}} function.}
}

\author{
	Joe Song and Haizhou Wang
}

\details{
Function \code{print.Ckmeans.1d.dp} and \code{print.Ckmedian.1d.dp} prints the maximum ratio of between-cluster sum of squares to total sum of squares unless all input elements are zero. The ratio is an indicator of maximum achievable clustering quality given the number of clusters: 100\% for a perfect clustering and 0\% for no cluster patterns.
}

\value{
	An object of class "\code{Ckmeans.1d.dp}" or "\code{Ckmedian.1d.dp}" as defined in \code{\link{Ckmeans.1d.dp}}.
}

\examples{
# Example: clustering data generated from a Gaussian
#          mixture model of two components
x <- c(rnorm(15, mean=-1, sd=0.3),
       rnorm(15, mean=1, sd=0.3))
res <- Ckmeans.1d.dp(x)
print(res)

res <- Ckmedian.1d.dp(x)
print(res)

y <- (rnorm(length(x)))^2
res <- Ckmeans.1d.dp(x, y=y)
print(res)

res <- Ckmedian.1d.dp(x)
print(res)
}

\keyword{ univar }
\keyword{ cluster }
\keyword{ print }

\name{plot.Cksegs.1d.dp}
\alias{plot.Cksegs.1d.dp}
\title{Plot Optimal Univariate Segmentation Results}
\description{
Plot optimal univariate segmentation results returned from \code{Cksegs.1d.dp}.
}
\usage{
\method{plot}{Cksegs.1d.dp}(x, xlab=NULL, ylab=NULL, main=NULL,
      sub=NULL, col.clusters=NULL, \dots)
}
%(x, xlab=ck$xname,
%     ylab=ifelse(x$yname=="1", "Weight", x$yname),
%     main="Optimal univariate clustering of", x$xname),
%     sub=paste("n =", length(x$cluster)),
%     col.clusters=seq_along(x$size),
%     \dots)

\arguments{
  \item{x}{an object of class as returned by \code{\link{Cksegs.1d.dp}}.}

 \item{xlab}{a character string. The x-axis label for the plot.}

  \item{ylab}{a character string. The x-axis label for the plot.}

 \item{main}{a character string. The title for the plot.}

  \item{sub}{a character string. The subtitle for the plot.}

  \item{col.clusters}{a vector of colors, defined either by integers or by color names. If the length is shorter than the number of clusters, the colors will be reused.}

  \item{...}{arguments passed to \code{\link{plot}} function in package \pkg{graphics}.}
}

\author{
	Joe Song
}

\details{
The function \code{plot.Cksegs.1d.dp} shows segments as horizontal lines from the univariate
segmentation results obtained from function \code{Cksegs.1d.dp}. It uses different colors to indicate segments.
}

\value{
	An object of class "\code{Cksegs.1d.dp}" defined in \code{\link{Cksegs.1d.dp}}.
}

\examples{
# Example: clustering data generated from a Gaussian
#          mixture model of three components
x <- c(rnorm(50, mean=-1, sd=0.3),
       rnorm(50, mean=1, sd=0.3),
       rnorm(50, mean=3, sd=0.3))

y <- x^3
res <- Cksegs.1d.dp(y, x=x)
plot(res, lwd=2)

}

\keyword{ univar }
\keyword{ cluster }
\keyword{ hplot }
\keyword{ distribution }

\name{plot.Cksegs.1d.dp}
\alias{plot.Cksegs.1d.dp}
\title{Plot Optimal Univariate Segmentation Results}
\description{
Plot optimal univariate segmentation results returned from \code{Cksegs.1d.dp}.
}
\usage{
\method{plot}{Cksegs.1d.dp}(x, xlab=NULL, ylab=NULL, main=NULL,
      sub=NULL, col.clusters=NULL, \dots)
}
%(x, xlab=ck$xname,
%     ylab=ifelse(x$yname=="1", "Weight", x$yname),
%     main="Optimal univariate clustering of", x$xname),
%     sub=paste("n =", length(x$cluster)),
%     col.clusters=seq_along(x$size),
%     \dots)

\arguments{
  \item{x}{an object of class as returned by \code{\link{Cksegs.1d.dp}}.}

 \item{xlab}{a character string. The x-axis label for the plot.}

  \item{ylab}{a character string. The x-axis label for the plot.}

 \item{main}{a character string. The title for the plot.}

  \item{sub}{a character string. The subtitle for the plot.}

  \item{col.clusters}{a vector of colors, defined either by integers or by color names. If the length is shorter than the number of clusters, the colors will be reused.}

  \item{...}{arguments passed to \code{\link{plot}} function in package \pkg{graphics}.}
}

\author{
	Joe Song
}

\details{
The function \code{plot.Cksegs.1d.dp} shows segments as horizontal lines from the univariate
segmentation results obtained from function \code{Cksegs.1d.dp}. It uses different colors to indicate segments.
}

\value{
	An object of class "\code{Cksegs.1d.dp}" defined in \code{\link{Cksegs.1d.dp}}.
}

\references{
  Wang, H. and Song, M. (2011) Ckmeans.1d.dp: optimal \var{k}-means clustering in one dimension by dynamic programming. \emph{The R Journal} \bold{3}(2), 29--33. Retrieved from \url{https://journal.r-project.org/archive/2011-2/RJournal_2011-2_Wang+Song.pdf}
}

\examples{
# Example: clustering data generated from a Gaussian
#          mixture model of three components
x <- c(rnorm(50, mean=-1, sd=0.3),
       rnorm(50, mean=1, sd=0.3),
       rnorm(50, mean=3, sd=0.3))

y <- x^3
res <- Cksegs.1d.dp(y, x=x)
plot(res, lwd=2)

}

\keyword{ univar }
\keyword{ cluster }
\keyword{ hplot }
\keyword{ distribution }
